@@ -52,9 +52,11 @@ import jnr.posix.Passwd;
 import jnr.posix.util.Platform;
 
 import org.jcodings.Encoding;
+import org.jcodings.specific.USASCIIEncoding;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.anno.JRubyClass;
 import org.jruby.exceptions.RaiseException;
+import org.jruby.runtime.Arity;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ClassIndex;
 import org.jruby.runtime.ThreadContext;
@@ -63,7 +65,9 @@ import org.jruby.util.*;
 import org.jruby.ast.util.ArgsUtil;
 
 import static org.jruby.RubyEnumerator.enumeratorize;
+import static org.jruby.RubyFile.filePathConvert;
 import static org.jruby.RubyString.UTF8;
+import static org.jruby.util.RubyStringBuilder.str;
 import static org.jruby.util.io.EncodingUtils.newExternalStringWithEncoding;
 
 /**
@@ -81,8 +85,6 @@ public class RubyDir extends RubyObject implements Closeable {
     private boolean isOpen = true;
     private Encoding encoding;
 
-    private static final Pattern PROTOCOL_PATTERN = Pattern.compile("^(uri|jar|file|classpath):([^:]*:)?//?.*");
-
     public RubyDir(Ruby runtime, RubyClass type) {
         super(runtime, type);
     }
@@ -99,13 +101,13 @@ public class RubyDir extends RubyObject implements Closeable {
         return dirClass;
     }
 
-    private final void checkDir() {
+    private void checkDir() {
         checkDirIgnoreClosed();
 
         if (!isOpen) throw getRuntime().newIOError("closed directory");
     }
 
-    private final void checkDirIgnoreClosed() {
+    private void checkDirIgnoreClosed() {
         testFrozen("Dir");
         // update snapshot (if changed) :
         if (snapshot == null || dir.exists() && dir.lastModified() > lastModified) {
@@ -180,11 +182,11 @@ public class RubyDir extends RubyObject implements Closeable {
 
 // ----- Ruby Class Methods ----------------------------------------------------
 
-    private static ArrayList<ByteList> dirGlobs(ThreadContext context, String cwd, IRubyObject[] args, int flags) {
+    private static ArrayList<ByteList> dirGlobs(ThreadContext context, String cwd, IRubyObject[] args, int flags, boolean sort) {
         ArrayList<ByteList> dirs = new ArrayList<>();
 
         for ( int i = 0; i < args.length; i++ ) {
-            dirs.addAll(Dir.push_glob(context.runtime, cwd, globArgumentAsByteList(context, args[i]), flags));
+            dirs.addAll(Dir.push_glob(context.runtime, cwd, globArgumentAsByteList(context, RubyFile.get_path(context, args[i])), flags, sort));
         }
 
         return dirs;
@@ -216,23 +218,42 @@ public class RubyDir extends RubyObject implements Closeable {
         }
     }
 
-    private static final String[] BASE = new String[] { "base" };
-    private static final String[] BASE_FLAGS = new String[] { "base", "flags" };
+    private static final String[] BASE_KEYWORDS = new String[] { "base", "sort" };
+    private static final String[] BASE_FLAGS_KEYWORDS = new String[] { "base", "sort", "flags" };
+
+    private static class GlobOptions {
+        String base = null;
+        int flags = 0;
+        boolean sort = true;
+    }
 
     // returns null (no kwargs present), "" kwargs but no base key, "something" kwargs with base key (which might be "").
-    private static String globOptions(ThreadContext context, IRubyObject[] args, int[] flags) {
+    private static void globOptions(ThreadContext context, IRubyObject[] args, String[] keys, GlobOptions options) {
         Ruby runtime = context.runtime;
 
         if (args.length > 1) {
             IRubyObject tmp = TypeConverter.checkHashType(runtime, args[args.length - 1]);
+            boolean processFlags = keys == BASE_FLAGS_KEYWORDS;
             if (tmp == context.nil) {
-                if (flags != null) {
-                    flags[0] = RubyNumeric.num2int(args[1]);
-                }
+                if (processFlags) options.flags = RubyNumeric.num2int(args[1]);
             } else {
-                String[] keys = flags != null ? BASE_FLAGS : BASE;
                 IRubyObject[] rets = ArgsUtil.extractKeywordArgs(context, (RubyHash) tmp, keys);
-                if (rets[0] == null || rets[0].isNil()) return "";
+
+                if (args.length == 3 && processFlags) options.flags = RubyNumeric.num2int(args[1]);
+                if (processFlags && rets[2] != null) options.flags |= RubyNumeric.num2int(rets[2]);
+
+                if (rets[1] != null) {
+                    if (!(rets[1] instanceof RubyBoolean)) {
+                        throw context.runtime.newArgumentError(str(runtime, "expected true or false as sort:", rets[1]));
+                    }
+                    options.sort = !runtime.getFalse().equals(rets[1]); // weirdly only explicit false is honored for sort.
+                }
+
+                if (rets[0] == null || rets[0].isNil()) {
+                    options.base = "";
+                    return;  
+                }
+
                 RubyString path = RubyFile.get_path(context, rets[0]);
                 Encoding[] enc = {path.getEncoding()};
                 String base = path.getUnicodeValue();
@@ -242,24 +263,22 @@ public class RubyDir extends RubyObject implements Closeable {
                     base = RubyFile.expandPath(context, base, enc, runtime.getCurrentDirectory(), true, false);
                 }
 
-                if (flags != null) flags[0] = rets[1] == null ? 0 : RubyNumeric.num2int(rets[1]);
-
-                return base;
+                options.base = base;
             }
         }
-
-        return null;
     }
 
-    @JRubyMethod(name = "[]", rest = true, meta = true)
+    @JRubyMethod(name = "[]", rest = true, meta = true, keywords = true)
     public static IRubyObject aref(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         Ruby runtime = context.runtime;
-        String base = globOptions(context, args, null);
+        GlobOptions options = new GlobOptions();
+        globOptions(context, args, BASE_KEYWORDS, options);
         List<ByteList> dirs;
+        String base = options.base;
 
         if (args.length == 1) {
             String dir = base == null || base.isEmpty() ? runtime.getCurrentDirectory() : base;
-            dirs = Dir.push_glob(runtime, dir, globArgumentAsByteList(context, args[0]), 0);
+            dirs = Dir.push_glob(runtime, dir, globArgumentAsByteList(context, args[0]), 0, options.sort);
         } else {
             IRubyObject[] arefArgs;
             if (base != null) { // kwargs are present
@@ -270,14 +289,26 @@ public class RubyDir extends RubyObject implements Closeable {
             }
 
             String dir = base.isEmpty() ? runtime.getCurrentDirectory() : base;
-            dirs = dirGlobs(context, dir, arefArgs, 0);
+            dirs = dirGlobs(context, dir, arefArgs, 0, options.sort);
         }
 
         return asRubyStringList(runtime, dirs);
     }
 
     private static ByteList globArgumentAsByteList(ThreadContext context, IRubyObject arg) {
-        return RubyFile.get_path(context, arg).getByteList();
+        RubyString str;
+        if (!(arg instanceof RubyString)) {
+            str = RubyFile.get_path(context, arg);
+        } else if (StringSupport.strNullCheck(arg)[0] == null) {
+            throw context.runtime.newArgumentError("nul-separated glob pattern is deprecated");
+        } else {
+            str = (RubyString) arg;
+            // FIXME: It is possible this can just be EncodingUtils.strCompatAndValid() but the spec says specifically it must be ascii compat which is more constrained than that method.
+            if (!str.getEncoding().isAsciiCompatible()) {
+                throw context.runtime.newEncodingCompatibilityError("incompatible character encodings: " + str.getEncoding() + " and " + USASCIIEncoding.INSTANCE);
+            }
+        }
+        return filePathConvert(context, str).getByteList();
     }
 
     /**
@@ -286,12 +317,15 @@ public class RubyDir extends RubyObject implements Closeable {
      * with each filename is passed to the block in turn. In this case, Nil is
      * returned.
      */
-    @JRubyMethod(required = 1, optional = 2, meta = true)
+    @JRubyMethod(required = 1, optional = 2, checkArity = false, meta = true)
     public static IRubyObject glob(ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
+        Arity.checkArgumentCount(context, args, 1, 3);
+
         Ruby runtime = context.runtime;
-        int[] flags = new int[] { 0 };
-        String base = globOptions(context, args, flags);
+        GlobOptions options = new GlobOptions();
+        globOptions(context, args, BASE_FLAGS_KEYWORDS, options);
         List<ByteList> dirs;
+        String base = options.base;
 
         if (base != null && !base.isEmpty() && !(JRubyFile.createResource(context, base).exists())){
             dirs = new ArrayList<ByteList>();
@@ -300,9 +334,9 @@ public class RubyDir extends RubyObject implements Closeable {
             String dir = base == null || base.isEmpty() ? runtime.getCurrentDirectory() : base;
 
             if (tmp.isNil()) {
-                dirs = Dir.push_glob(runtime, dir, globArgumentAsByteList(context, args[0]), flags[0]);
+                dirs = Dir.push_glob(runtime, dir, globArgumentAsByteList(context, args[0]), options.flags, options.sort);
             } else {
-                dirs = dirGlobs(context, dir, ((RubyArray) tmp).toJavaArray(), flags[0]);
+                dirs = dirGlobs(context, dir, ((RubyArray) tmp).toJavaArray(), options.flags, options.sort);
             }
         }
 
@@ -398,10 +432,11 @@ public class RubyDir extends RubyObject implements Closeable {
     }
 
     /** Changes the current directory to <code>path</code> */
-    @JRubyMethod(optional = 1, meta = true)
+    @JRubyMethod(optional = 1, checkArity = false, meta = true)
     public static IRubyObject chdir(ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
+        int argc = Arity.checkArgumentCount(context, args, 0, 1);
         Ruby runtime = context.runtime;
-        RubyString path = args.length == 1 ?
+        RubyString path = argc == 1 ?
             StringSupport.checkEmbeddedNulls(runtime, RubyFile.get_path(context, args[0])) :
             getHomeDirectoryPath(context);
 
@@ -432,7 +467,7 @@ public class RubyDir extends RubyObject implements Closeable {
     /**
      * Changes the root directory (only allowed by super user).  Not available on all platforms.
      */
-    @JRubyMethod(name = "chroot", required = 1, meta = true)
+    @JRubyMethod(name = "chroot", meta = true, notImplemented = true)
     public static IRubyObject chroot(IRubyObject recv, IRubyObject path) {
         throw recv.getRuntime().newNotImplementedError("chroot not implemented: chroot is non-portable and is not supported.");
     }
@@ -464,7 +499,7 @@ public class RubyDir extends RubyObject implements Closeable {
         return rmdir19(recv.getRuntime().getCurrentContext(), recv, path);
     }
 
-    @JRubyMethod(name = {"rmdir", "unlink", "delete"}, required = 1, meta = true)
+    @JRubyMethod(name = {"rmdir", "unlink", "delete"}, meta = true)
     public static IRubyObject rmdir19(ThreadContext context, IRubyObject recv, IRubyObject path) {
         Ruby runtime = context.runtime;
         RubyString cleanPath = StringSupport.checkEmbeddedNulls(runtime, RubyFile.get_path(context, path));
@@ -586,9 +621,7 @@ public class RubyDir extends RubyObject implements Closeable {
     public static RubyString getwd(IRubyObject recv) {
         Ruby runtime = recv.getRuntime();
 
-        RubyString pwd = newFilesystemString(runtime, getCWD(runtime));
-        pwd.setTaint(true);
-        return pwd;
+        return newFilesystemString(runtime, getCWD(runtime));
     }
 
     /**
@@ -601,7 +634,15 @@ public class RubyDir extends RubyObject implements Closeable {
 
     @JRubyMethod(name = "home", meta = true)
     public static IRubyObject home(ThreadContext context, IRubyObject recv, IRubyObject user) {
-        return getHomeDirectoryPath(context, user.convertToString().toString());
+        if (user == null || user.isNil()) {
+            return getHomeDirectoryPath(context);
+        }
+
+        RubyString userString = user.convertToString();
+
+        userString.verifyAsciiCompatible();
+
+        return getHomeDirectoryPath(context, userString.toString());
     }
 
     /**
@@ -609,8 +650,9 @@ public class RubyDir extends RubyObject implements Closeable {
      * <code>mode</code> parameter is provided only to support existing Ruby
      * code, and is ignored.
      */
-    @JRubyMethod(name = "mkdir", required = 1, optional = 1, meta = true)
+    @JRubyMethod(name = "mkdir", required = 1, optional = 1, checkArity = false, meta = true)
     public static IRubyObject mkdir(ThreadContext context, IRubyObject recv, IRubyObject... args) {
+        Arity.checkArgumentCount(context, args, 1, 2);
         Ruby runtime = context.runtime;
         RubyString path = StringSupport.checkEmbeddedNulls(runtime, RubyFile.get_path(context, args[0]));
         return mkdirCommon(runtime, path.asJavaString(), args);
@@ -812,7 +854,7 @@ public class RubyDir extends RubyObject implements Closeable {
      * returned by <code>tell</code> or 0.
      */
 
-    @JRubyMethod(name = "seek", required = 1)
+    @JRubyMethod(name = "seek")
     public IRubyObject seek(IRubyObject newPos) {
         checkDir();
 
@@ -820,7 +862,7 @@ public class RubyDir extends RubyObject implements Closeable {
         return this;
     }
 
-    @JRubyMethod(name = "pos=", required = 1)
+    @JRubyMethod(name = "pos=")
     public IRubyObject set_pos(IRubyObject newPos) {
         int pos2 = RubyNumeric.fix2int(newPos);
         if (pos2 >= 0) this.pos = pos2;
@@ -850,7 +892,7 @@ public class RubyDir extends RubyObject implements Closeable {
         final String[] snapshot = this.snapshot;
         if (pos >= snapshot.length) return getRuntime().getNil();
 
-        RubyString result = RubyString.newString(getRuntime(), snapshot[pos], encoding);
+        RubyString result = newExternalStringWithEncoding(getRuntime(), snapshot[pos], encoding);
         pos++;
         return result;
     }
@@ -892,7 +934,7 @@ public class RubyDir extends RubyObject implements Closeable {
     @JRubyMethod(name = "exists?", meta = true)
     public static IRubyObject exists_p(ThreadContext context, IRubyObject recv, IRubyObject arg) {
         if (context.runtime.warningsEnabled()) {
-            context.runtime.getWarnings().warn("Dir.exists? is a deprecated name, use Dir.exist? instead");
+            context.runtime.getWarnings().warnDeprecatedAlternate("Dir.exists?", "Dir.exist?");
         }
 
         return exist(context, recv, arg);
