@@ -222,6 +222,10 @@ public class RubyFixnum extends RubyInteger implements Constantizable {
         return new RubyFixnum(runtime, value);
     }
 
+    public static RubyFixnum newFixnum(Ruby runtime, int value) {
+        return newFixnum(runtime, (long) value);
+    }
+
     private static boolean isInCacheRange(long value) {
         return value <= CACHE_OFFSET - 1 && value >= -CACHE_OFFSET;
     }
@@ -626,11 +630,11 @@ public class RubyFixnum extends RubyInteger implements Constantizable {
 
     /** fix_div
      * here is terrible MRI gotcha:
-     * 1.div 3.0 -> 0
-     * 1 / 3.0   -> 0.3333333333333333
+     * 1.div 3.0$ -&gt; 0
+     * 1 / 3.0  $ -&gt; 0.3333333333333333
      *
      * MRI is also able to do it in one place by looking at current frame in rb_num_coerce_bin:
-     * rb_funcall(x, ruby_frame->orig_func, 1, y);
+     * rb_funcall(x, ruby_frame-&gt;orig_func, 1, y);
      *
      * also note that RubyFloat doesn't override Numeric.div
      */
@@ -996,6 +1000,23 @@ public class RubyFixnum extends RubyInteger implements Constantizable {
         return RubyBoolean.newBoolean(context, (double) value == other);
     }
 
+    /** fix_not_equal
+     *
+     */
+    @Override
+    public IRubyObject op_not_equal(ThreadContext context, IRubyObject other) {
+        return other instanceof RubyFixnum ?
+                op_not_equal(context, ((RubyFixnum) other).value) : super.op_not_equal(context, other);
+    }
+
+    public IRubyObject op_not_equal(ThreadContext context, long other) {
+        return RubyBoolean.newBoolean(context, value != other);
+    }
+
+    public IRubyObject op_not_equal(ThreadContext context, double other) {
+        return RubyBoolean.newBoolean(context, (double) value != other);
+    }
+
     public boolean op_equal_boolean(ThreadContext context, long other) {
         return value == other;
     }
@@ -1252,11 +1273,11 @@ public class RubyFixnum extends RubyInteger implements Constantizable {
         return newFixnum(context.runtime, value ^ other);
     }
 
-    /** fix_aref
+    /** rb_fix_aref
      *
      */
     @Override
-    public IRubyObject op_aref(ThreadContext context, IRubyObject other) {
+    protected IRubyObject op_aref_subclass(ThreadContext context, IRubyObject other) {
         if (!(other instanceof RubyFixnum) && !((other = fixCoerce(other)) instanceof RubyFixnum)) {
             BigInteger big = ((RubyBignum) other).value;
             other = RubyBignum.bignorm(context.runtime, big);
@@ -1411,11 +1432,6 @@ public class RubyFixnum extends RubyInteger implements Constantizable {
         return super.id();
     }
 
-    @Override
-    public IRubyObject taint(ThreadContext context) {
-        return this;
-    }
-
     // Piece of mri rb_to_id
     @Override
     public String asJavaString() {
@@ -1515,11 +1531,14 @@ public class RubyFixnum extends RubyInteger implements Constantizable {
         return context.sites.Fixnum;
     }
 
-    /** rb_fix_induced_from
-     *
-     */
     @Deprecated
     public static IRubyObject induced_from(IRubyObject recv, IRubyObject other) {
         return RubyNumeric.num2fix(other);
     }
+
+    @Deprecated
+    @Override
+    public IRubyObject taint(ThreadContext context) {
+        return this;
+    }
 }
