@@ -44,13 +44,13 @@ import jnr.posix.POSIX;
 import org.jcodings.Encoding;
 import org.jcodings.specific.USASCIIEncoding;
 import org.jruby.anno.JRubyMethod;
+import org.jruby.ast.util.ArgsUtil;
 import org.jruby.common.IRubyWarnings.ID;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.internal.runtime.GlobalVariables;
 import org.jruby.internal.runtime.ValueAccessor;
 import org.jruby.ir.Tuple;
 import org.jruby.javasupport.JavaUtil;
-import org.jruby.runtime.Helpers;
 import org.jruby.platform.Platform;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.Constants;
@@ -102,7 +102,7 @@ public class RubyGlobal {
         String[] argv = runtime.getInstanceConfig().getArgv();
 
         for (String arg : argv) {
-            argvArray.append(RubyString.newInternalFromJavaExternal(runtime, arg).tainted());
+            argvArray.append(RubyString.newInternalFromJavaExternal(runtime, arg));
         }
 
         if (runtime.getObject().getConstantNoConstMissing("ARGV") != null) {
@@ -116,15 +116,8 @@ public class RubyGlobal {
     public static void createGlobals(Ruby runtime) {
         GlobalVariables globals = runtime.getGlobalVariables();
 
-        runtime.defineGlobalConstant("TOPLEVEL_BINDING", runtime.newBinding());
-
-        runtime.defineGlobalConstant("TRUE", runtime.getTrue());
-        runtime.defineGlobalConstant("FALSE", runtime.getFalse());
-        runtime.defineGlobalConstant("NIL", runtime.getNil());
-
-        runtime.getObject().deprecateConstant(runtime, "TRUE");
-        runtime.getObject().deprecateConstant(runtime, "FALSE");
-        runtime.getObject().deprecateConstant(runtime, "NIL");
+        runtime.setTopLevelBinding(runtime.newBinding());
+        runtime.defineGlobalConstant("TOPLEVEL_BINDING", runtime.getTopLevelBinding());
 
         initARGV(runtime);
 
@@ -168,20 +161,13 @@ public class RubyGlobal {
         release.setFrozen(true);
         runtime.defineGlobalConstant("JRUBY_VERSION", jrubyVersion);
         runtime.defineGlobalConstant("JRUBY_REVISION", jrubyRevision);
-
-        // needs to be a fixnum, but our revision is a sha1 hash from git
-        runtime.defineGlobalConstant("RUBY_REVISION", runtime.newFixnum(Constants.RUBY_REVISION));
+        runtime.defineGlobalConstant("RUBY_REVISION", runtime.newString(Constants.REVISION));
         runtime.defineGlobalConstant("RUBY_ENGINE", engine);
         runtime.defineGlobalConstant("RUBY_ENGINE_VERSION", jrubyVersion);
 
         RubyInstanceConfig.Verbosity verbosity = runtime.getInstanceConfig().getVerbosity();
         runtime.defineVariable(new WarningGlobalVariable(runtime, "$-W", verbosity), GLOBAL);
 
-        final GlobalVariable kcodeGV;
-        kcodeGV = new NonEffectiveGlobalVariable(runtime, "$KCODE", runtime.getNil());
-
-        runtime.defineVariable(kcodeGV, GLOBAL);
-        runtime.defineVariable(new GlobalVariable.Copy(runtime, "$-K", kcodeGV), GLOBAL);
         IRubyObject defaultRS = runtime.newString(runtime.getInstanceConfig().getRecordSeparator());
         release.setFrozen(true);
         GlobalVariable rs = new StringGlobalVariable(runtime, "$/", defaultRS);
@@ -189,7 +175,7 @@ public class RubyGlobal {
         runtime.setRecordSeparatorVar(rs);
         globals.setDefaultSeparator(defaultRS);
         runtime.defineVariable(new StringGlobalVariable(runtime, "$\\", runtime.getNil()), GLOBAL);
-        runtime.defineVariable(new StringGlobalVariable(runtime, "$,", runtime.getNil()), GLOBAL);
+        runtime.defineVariable(new DeprecatedStringGlobalVariable(runtime, "$,", runtime.getNil()), GLOBAL);
 
         runtime.defineVariable(new LineNumberGlobalVariable(runtime, "$."), GLOBAL);
         runtime.defineVariable(new LastlineGlobalVariable(runtime, "$_"), FRAME);
@@ -199,9 +185,9 @@ public class RubyGlobal {
         runtime.defineVariable(new NonEffectiveGlobalVariable(runtime, "$=", runtime.getFalse()), GLOBAL);
 
         if(runtime.getInstanceConfig().getInputFieldSeparator() == null) {
-            runtime.defineVariable(new StringOrRegexpGlobalVariable(runtime, "$;", runtime.getNil()), GLOBAL);
+            runtime.defineVariable(new DeprecatedStringOrRegexpGlobalVariable(runtime, "$;", runtime.getNil()), GLOBAL);
         } else {
-            runtime.defineVariable(new StringOrRegexpGlobalVariable(runtime, "$;", RubyRegexp.newRegexp(runtime, runtime.getInstanceConfig().getInputFieldSeparator(), new RegexpOptions())), GLOBAL);
+            runtime.defineVariable(new DeprecatedStringOrRegexpGlobalVariable(runtime, "$;", RubyRegexp.newRegexp(runtime, runtime.getInstanceConfig().getInputFieldSeparator(), new RegexpOptions())), GLOBAL);
         }
 
         RubyInstanceConfig.Verbosity verbose = runtime.getInstanceConfig().getVerbosity();
@@ -213,20 +199,19 @@ public class RubyGlobal {
         } else {
             verboseValue = runtime.getFalse();
         }
-        runtime.defineVariable(new VerboseGlobalVariable(runtime, "$VERBOSE", verboseValue), GLOBAL);
-        runtime.defineVariable(new VerboseGlobalVariable(runtime, "$-v", verboseValue), GLOBAL);
-        runtime.defineVariable(new VerboseGlobalVariable(runtime, "$-w", verboseValue), GLOBAL);
+        runtime.setVerbose(verboseValue);
+        runtime.defineVariable(new VerboseGlobalVariable(runtime, "$VERBOSE"), GLOBAL);
+        runtime.defineVariable(new VerboseGlobalVariable(runtime, "$-v"), GLOBAL);
+        runtime.defineVariable(new VerboseGlobalVariable(runtime, "$-w"), GLOBAL);
 
-        IRubyObject debug = runtime.newBoolean(runtime.getInstanceConfig().isDebug());
-        runtime.defineVariable(new DebugGlobalVariable(runtime, "$DEBUG", debug), GLOBAL);
-        runtime.defineVariable(new DebugGlobalVariable(runtime, "$-d", debug), GLOBAL);
-
-        runtime.defineVariable(new SafeGlobalVariable(runtime, "$SAFE"), THREAD);
+        runtime.setDebug(runtime.newBoolean(runtime.getInstanceConfig().isDebug()));
+        runtime.defineVariable(new DebugGlobalVariable(runtime, "$DEBUG"), GLOBAL);
+        runtime.defineVariable(new DebugGlobalVariable(runtime, "$-d"), GLOBAL);
 
         runtime.defineVariable(new BacktraceGlobalVariable(runtime, "$@"), THREAD);
 
         initSTDIO(runtime, globals);
-
+        
         runtime.defineVariable(new LoadedFeatures(runtime, "$\""), GLOBAL);
         runtime.defineVariable(new LoadedFeatures(runtime, "$LOADED_FEATURES"), GLOBAL);
 
@@ -256,7 +241,7 @@ public class RubyGlobal {
         globals.defineReadonly("$-p",
                 new ValueAccessor(runtime.newBoolean(runtime.getInstanceConfig().isAssumePrinting())),
                 GLOBAL);
-        globals.defineReadonly("$-a",
+                globals.defineReadonly("$-a",
                 new ValueAccessor(runtime.newBoolean(runtime.getInstanceConfig().isSplit())),
                 GLOBAL);
         globals.defineReadonly("$-l",
@@ -266,6 +251,13 @@ public class RubyGlobal {
         // ARGF, $< object
         RubyArgsFile.initArgsFile(runtime);
 
+        String inplace = runtime.config.getInPlaceBackupExtension();
+        if (inplace != null) {
+            runtime.defineVariable(new ArgfGlobalVariable(runtime, "$-i", runtime.newString(inplace)), GLOBAL);
+        } else {
+            runtime.defineVariable(new ArgfGlobalVariable(runtime, "$-i", runtime.getNil()), GLOBAL);
+        }
+
         globals.alias("$-0", "$/");
 
         // Define aliases originally in the "English.rb" stdlib
@@ -418,6 +410,10 @@ public class RubyGlobal {
         runtime.defineGlobalConstant("ENV_JAVA", envJava);
     }
 
+    private static void warnDeprecatedGlobal(final Ruby runtime, final String name) {
+        runtime.getWarnings().warnDeprecated(ID.MISCELLANEOUS, "`" + name + "' is deprecated");
+    }
+
     /**
      * Obligate string-keyed and string-valued hash, used for ENV.
      * On Windows, the keys are case-insensitive for ENV
@@ -431,7 +427,7 @@ public class RubyGlobal {
 
         protected final boolean isCaseSensitive() { return false; }
 
-        @JRubyMethod(name = "[]", required = 1)
+        @JRubyMethod(name = "[]")
         public IRubyObject op_aref(ThreadContext context, IRubyObject arg) {
             IRubyObject key = arg.convertToString();
             IRubyObject value = internalGet(key);
@@ -446,6 +442,12 @@ public class RubyGlobal {
 
         private static final ByteList ENV = new ByteList(new byte[] {'E','N','V'}, USASCIIEncoding.INSTANCE, false);
 
+        @JRubyMethod
+        public IRubyObject freeze(ThreadContext context) {
+            // FIXME: So far I can see this is only used by ENV so I put it here but perhaps we need to differentiate case
+            throw context.runtime.newTypeError("cannot freeze ENV");
+        }
+
         @JRubyMethod(name = "assoc")
         public IRubyObject assoc(final ThreadContext context, IRubyObject obj) {
             return super.assoc(context, verifyStringLike(context, obj).convertToString());
@@ -484,12 +486,12 @@ public class RubyGlobal {
             return super.rassoc(context, obj.convertToString());
         }
 
-        @JRubyMethod(name = {"has_key?", "key?", "include?", "member?"}, required = 1)
+        @JRubyMethod(name = {"has_key?", "key?", "include?", "member?"})
         public RubyBoolean has_key_p(ThreadContext context, IRubyObject key) {
             return internalGetEntry(verifyStringLike(context, key)) == NO_ENTRY ? context.fals : context.tru;
         }
 
-        @JRubyMethod(name = {"has_value?", "value?"}, required = 1)
+        @JRubyMethod(name = {"has_value?", "value?"})
         public IRubyObject has_value_pp(ThreadContext context, IRubyObject expected) {
             if (!isStringLike(expected)) return context.nil;
 
@@ -537,7 +539,7 @@ public class RubyGlobal {
             return context.nil;
         }
 
-        @JRubyMethod(name = "replace", required = 1)
+        @JRubyMethod(name = "replace")
         public RubyHash replace(final ThreadContext context, IRubyObject other) {
             modify();
 
@@ -606,10 +608,46 @@ public class RubyGlobal {
             return block.isGiven() ? h.to_h_block(context, block) : h;
         }
 
+        @JRubyMethod(name = "clone")
+        @Override
+        public IRubyObject rbClone(ThreadContext context) {
+            context.runtime.getWarnings().warnDeprecatedAlternate("ENV.clone", "ENV.to_h");
+
+            return this;
+        }
+
+        @JRubyMethod(name = "clone")
+        public IRubyObject rbClone(ThreadContext context, IRubyObject _opts) {
+            Ruby runtime = context.runtime;
+
+            IRubyObject opts = ArgsUtil.getOptionsArg(runtime, _opts, true);
+            if (opts.isNil()) {
+                throw runtime.newArgumentError(1, 0);
+            }
+
+            IRubyObject freeze = ArgsUtil.getFreezeOpt(context, opts);
+            if (freeze != null && freeze.isTrue()) {
+                throw runtime.newTypeError("cannot freeze ENV");
+            }
+
+            return rbClone(context);
+        }
+
+        @JRubyMethod()
+        @Override
+        public IRubyObject dup(ThreadContext context) {
+            throw context.runtime.newTypeError("Cannot dup ENV, use ENV.to_h to get a copy of ENV as a hash");
+        }
+
         private final RaiseException concurrentModification() {
             return metaClass.runtime.newConcurrencyError(
                     "Detected invalid hash contents due to unsynchronized modifications with concurrent users");
         }
+
+        @Override
+        protected void replaceWith(ThreadContext context, RubyHash otherHash) {
+            replaceExternally(context, otherHash);
+        }
     }
 
     /**
@@ -693,11 +731,12 @@ public class RubyGlobal {
         }
 
         protected static IRubyObject verifyStringLike(ThreadContext context, IRubyObject test) {
-            if (!isStringLike(test)) {
+            IRubyObject string = test.checkStringType();
+            if (string.isNil()) {
                 throw context.runtime.newTypeError("no implicit conversion of " + test.getMetaClass() + " into String");
             }
 
-            return test;
+            return string;
         }
 
         private static RubyString verifyValidKey(ThreadContext context, RubyString key, IRubyObject value) {
@@ -734,25 +773,19 @@ public class RubyGlobal {
             return actualKey;
         }
 
-        private static final ByteList PATH_BYTES = new ByteList(new byte[] {'P','A','T','H'}, USASCIIEncoding.INSTANCE, false);
-
         // MRI: env_name_new
+        // TODO: mri 3.1 does not use env_name_new
         protected static IRubyObject newName(ThreadContext context, IRubyObject key, IRubyObject valueArg) {
             if (valueArg.isNil()) return context.nil;
 
             RubyString value = (RubyString) valueArg;
-            EncodingService encodingService = context.runtime.getEncodingService();
-            Encoding encoding = isPATH(context, (RubyString) key) && !value.isTaint() ?
-                    encodingService.getFileSystemEncoding() :
-                    encodingService.getLocaleEncoding();
 
-            return newString(context, value, encoding);
+            return newString(context, value);
         }
 
         protected static IRubyObject newString(ThreadContext context, RubyString value, Encoding encoding) {
             IRubyObject result = newExternalStringWithEncoding(context.runtime, value.strDup(context.runtime).getByteList().shallowDup(), encoding);
 
-            result.setTaint(true);
             result.setFrozen(true);
 
             return result;
@@ -760,7 +793,7 @@ public class RubyGlobal {
 
         // MRI: env_str_new
         protected static IRubyObject newString(ThreadContext context, RubyString value) {
-            return newString(context, value, context.runtime.getEncodingService().getLocaleEncoding());
+            return newString(context, value, context.runtime.getEncodingService().getEnvEncoding());
         }
 
         // MRI: env_str_new2
@@ -768,12 +801,6 @@ public class RubyGlobal {
             return obj.isNil() ? context.nil : newString(context, (RubyString) obj);
         }
 
-        private static boolean isPATH(ThreadContext context, RubyString name) {
-            return Platform.IS_WINDOWS ?
-                    equalIgnoreCase(context, name, RubyString.newString(context.runtime, PATH_BYTES)) :
-                    name.getByteList().equal(PATH_BYTES);
-        }
-
         private static boolean equalIgnoreCase(ThreadContext context, final RubyString str1, final RubyString str2) {
             return ((RubyFixnum) str1.casecmp(context, str2)).isZero();
         }
@@ -803,13 +830,13 @@ public class RubyGlobal {
 
         @Override
         public IRubyObject set(IRubyObject value) {
-            runtime.getWarnings().warn(ID.INEFFECTIVE_GLOBAL, "warning: variable " + name + " is no longer effective; ignored");
+            runtime.getWarnings().warnDeprecated(ID.INEFFECTIVE_GLOBAL, "warning: variable " + name + " is no longer effective; ignored");
             return value;
         }
 
         @Override
         public IRubyObject get() {
-            runtime.getWarnings().warn(ID.INEFFECTIVE_GLOBAL, "warning: variable " + name + " is no longer effective");
+            runtime.getWarnings().warnDeprecated(ID.INEFFECTIVE_GLOBAL, "warning: variable " + name + " is no longer effective");
             return value;
         }
     }
@@ -958,6 +985,52 @@ public class RubyGlobal {
         }
     }
 
+    public static class ArgfGlobalVariable extends GlobalVariable {
+        public ArgfGlobalVariable(Ruby runtime, String name, IRubyObject value) {
+            super(runtime, name, value);
+            set(value);
+        }
+
+        @Override
+        public IRubyObject set(IRubyObject value) {
+            RubyArgsFile.inplace_mode_set(runtime.getCurrentContext(), runtime.getArgsFile(), value);
+            if (value.isNil() || !value.isTrue()) {
+                runtime.getInstanceConfig().setInPlaceBackupExtension(null);
+            }
+            return super.set(value);
+        }
+    }
+
+    public static class DeprecatedStringGlobalVariable extends StringGlobalVariable {
+        public DeprecatedStringGlobalVariable(Ruby runtime, String name, IRubyObject value) {
+            super(runtime, name, value);
+        }
+
+        @Override
+        public IRubyObject set(IRubyObject value) {
+            IRubyObject result = super.set(value);
+
+            if (!value.isNil()) warnDeprecatedGlobal(runtime, name);
+
+            return result;
+        }
+    }
+
+    public static class DeprecatedStringOrRegexpGlobalVariable extends StringOrRegexpGlobalVariable {
+        public DeprecatedStringOrRegexpGlobalVariable(Ruby runtime, String name, IRubyObject value) {
+            super(runtime, name, value);
+        }
+
+        @Override
+        public IRubyObject set(IRubyObject value) {
+            IRubyObject result = super.set(value);
+
+            if (!result.isNil()) warnDeprecatedGlobal(runtime, name);
+
+            return result;
+        }
+    }
+
     public static class StringOrRegexpGlobalVariable extends GlobalVariable {
         public StringOrRegexpGlobalVariable(Ruby runtime, String name, IRubyObject value) {
             super(runtime, name, value);
@@ -990,27 +1063,9 @@ public class RubyGlobal {
         }
     }
 
-    private static class SafeGlobalVariable extends GlobalVariable {
-        public SafeGlobalVariable(Ruby runtime, String name) {
-            super(runtime, name, null);
-        }
-
-        @Override
-        public IRubyObject get() {
-            return RubyFixnum.zero(runtime);
-        }
-
-        @Override
-        public IRubyObject set(IRubyObject value) {
-            runtime.getWarnings().warnOnce(ID.SAFE_NOT_SUPPORTED, "SAFE levels are not supported in JRuby");
-            return RubyFixnum.zero(runtime);
-        }
-    }
-
     private static class VerboseGlobalVariable extends GlobalVariable {
-        public VerboseGlobalVariable(Ruby runtime, String name, IRubyObject initialValue) {
-            super(runtime, name, initialValue);
-            set(initialValue);
+        public VerboseGlobalVariable(Ruby runtime, String name) {
+            super(runtime, name, null); // this.value not used
         }
 
         @Override
@@ -1020,11 +1075,7 @@ public class RubyGlobal {
 
         @Override
         public IRubyObject set(IRubyObject newValue) {
-            if (newValue.isNil()) {
-                runtime.setVerbose(newValue);
-            } else {
-                runtime.setVerbose(runtime.newBoolean(newValue.isTrue()));
-            }
+            runtime.setVerbose(newValue.isNil() ? null : newValue.isTrue());
 
             return newValue;
         }
@@ -1042,9 +1093,8 @@ public class RubyGlobal {
     }
 
     private static class DebugGlobalVariable extends GlobalVariable {
-        public DebugGlobalVariable(Ruby runtime, String name, IRubyObject initialValue) {
-            super(runtime, name, initialValue);
-            set(initialValue);
+        public DebugGlobalVariable(Ruby runtime, String name) {
+            super(runtime, name, null); // this.value not used
         }
 
         @Override
@@ -1054,11 +1104,7 @@ public class RubyGlobal {
 
         @Override
         public IRubyObject set(IRubyObject newValue) {
-            if (newValue.isNil()) {
-                runtime.setDebug(newValue);
-            } else {
-                runtime.setDebug(runtime.newBoolean(newValue.isTrue()));
-            }
+            runtime.setDebug(newValue.isTrue());
 
             return newValue;
         }
