@@ -31,6 +31,7 @@ package org.jruby;
 
 import jnr.posix.util.Platform;
 
+import org.jruby.common.RubyWarnings;
 import org.jruby.exceptions.MainExitException;
 import org.jruby.runtime.Constants;
 import org.jruby.runtime.backtrace.TraceType;
@@ -57,6 +58,8 @@ import java.io.*;
 import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Collections;
+import java.util.EnumSet;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.LinkedHashSet;
@@ -164,6 +167,8 @@ public class RubyInstanceConfig {
     }
 
     public void processArgumentsWithRubyopts() {
+        if (disableRUBYOPT) return;
+        
         // environment defaults to System.getenv normally
         Object rubyoptObj = environment.get("RUBYOPT");
 
@@ -901,6 +906,14 @@ public class RubyInstanceConfig {
         this.verbosity = verbosity;
     }
 
+    public void setBacktraceLimit(Integer limit) {
+        this.backtraceLimit = limit;
+    }
+
+    public Integer getBacktraceLimit() {
+        return this.backtraceLimit;
+    }
+
     /**
      * @see Options#CLI_VERBOSE
      */
@@ -922,6 +935,15 @@ public class RubyInstanceConfig {
         this.debug = debug;
     }
 
+    /**
+     * Get the set of enabled warning categories.
+     *
+     * @return the set of enabled warning categories
+     */
+    public Set<RubyWarnings.Category> getWarningCategories() {
+        return warningCategories;
+    }
+
     /**
      * @see Options#CLI_PARSER_DEBUG
      */
@@ -1533,10 +1555,10 @@ public class RubyInstanceConfig {
     private boolean shouldRunInterpreter = true;
     private boolean shouldPrintUsage = Options.CLI_HELP.load();
     private boolean shouldPrintProperties=Options.CLI_PROPERTIES.load();
-    private boolean dumpConfig=false;
     private KCode kcode = Options.CLI_KCODE.load();
     private String recordSeparator = Options.CLI_RECORD_SEPARATOR.load();
     private boolean shouldCheckSyntax = Options.CLI_CHECK_SYNTAX.load();
+    private Integer backtraceLimit = Options.CLI_BACKTRACE_LIMIT.load();
     private String inputFieldSeparator = Options.CLI_AUTOSPLIT_SEPARATOR.load();
     private boolean managementEnabled = false;
     private String inPlaceBackupExtension = Options.CLI_BACKUP_EXTENSION.load();
@@ -1578,6 +1600,8 @@ public class RubyInstanceConfig {
 
     private boolean forceStdin = false;
 
+    private final Set<RubyWarnings.Category> warningCategories = Collections.synchronizedSet(EnumSet.of(RubyWarnings.Category.EXPERIMENTAL));
+
     ////////////////////////////////////////////////////////////////////////////
     // Support classes, etc.
     ////////////////////////////////////////////////////////////////////////////
@@ -1624,49 +1648,42 @@ public class RubyInstanceConfig {
      */
     public static final int JAVA_VERSION = initJavaBytecodeVersion();
 
-    /**
-     * Enable fast operator compiler optimizations.
-     *
-     * Set with the <tt>jruby.compile.fastops</tt> system property.
-     */
-    public static final boolean FASTOPS_COMPILE_ENABLED = Options.COMPILE_FASTOPS.load();
-
     /**
      * Enable a thread pool. Each Ruby thread will be mapped onto a thread from this pool.
      *
-     * Set with the <tt>jruby.thread.pool.enabled</tt> system property.
+     * Set with the <code>jruby.thread.pool.enabled</code> system property.
      */
     public static final boolean POOLING_ENABLED = false;
 
     /**
      * Maximum thread pool size (integer, default Integer.MAX_VALUE).
      *
-     * Set with the <tt>jruby.thread.pool.max</tt> system property.
+     * Set with the <code>jruby.thread.pool.max</code> system property.
      */
     public static final int POOL_MAX = Options.THREADPOOL_MAX.load();
     /**
      * Minimum thread pool size (integer, default 0).
      *
-     * Set with the <tt>jruby.thread.pool.min</tt> system property.
+     * Set with the <code>jruby.thread.pool.min</code> system property.
      */
     public static final int POOL_MIN = Options.THREADPOOL_MIN.load();
     /**
      * Thread pool time-to-live in seconds.
      *
-     * Set with the <tt>jruby.thread.pool.max</tt> system property.
+     * Set with the <code>jruby.thread.pool.max</code> system property.
      */
     public static final int POOL_TTL = Options.THREADPOOL_TTL.load();
     /**
      * Fiber thread pool time-to-live in seconds.
      *
-     * Set with the <tt>jruby.fiber.thread.pool.max</tt> system property.
+     * Set with the <code>jruby.fiber.thread.pool.max</code> system property.
      */
     public static final int FIBER_POOL_TTL = Options.FIBER_THREADPOOL_TTL.load();
 
     /**
      * Enable tracing of method calls.
      *
-     * Set with the <tt>jruby.debug.fullTrace</tt> system property.
+     * Set with the <code>jruby.debug.fullTrace</code> system property.
      */
     public static boolean FULL_TRACE_ENABLED = Options.DEBUG_FULLTRACE.load();
 
@@ -1676,7 +1693,7 @@ public class RubyInstanceConfig {
      *
      * Also supports excluding based on implementation_file.rb syntax.
      *
-     * Set with the <tt>jruby.jit.exclude</tt> system property.
+     * Set with the <code>jruby.jit.exclude</code> system property.
      */
     public static final String COMPILE_EXCLUDE = Options.JIT_EXCLUDE.load();
 
@@ -1684,7 +1701,7 @@ public class RubyInstanceConfig {
      * Indicates the global default for whether native code is enabled. Default
      * is true. This value is used to default new runtime configurations.
      *
-     * Set with the <tt>jruby.native.enabled</tt> system property.
+     * Set with the <code>jruby.native.enabled</code> system property.
      */
     public static final boolean NATIVE_ENABLED = Options.NATIVE_ENABLED.load();
 
@@ -1694,42 +1711,42 @@ public class RubyInstanceConfig {
     /**
      * Whether to reify (pre-compile and generate) a Java class per Ruby class.
      *
-     * Set with the <tt>jruby.reify.classes</tt> system property.
+     * Set with the <code>jruby.reify.classes</code> system property.
      */
     public static final boolean REIFY_RUBY_CLASSES = Options.REIFY_CLASSES.load();
 
     /**
      * Log errors that occur during reification.
      *
-     * Set with the <tt>jruby.reify.logErrors</tt> system property.
+     * Set with the <code>jruby.reify.logErrors</code> system property.
      */
     public static final boolean REIFY_LOG_ERRORS = Options.REIFY_LOGERRORS.load();
 
     /**
      * Turn on debugging of the load service (requires and loads).
      *
-     * Set with the <tt>jruby.debug.loadService</tt> system property.
+     * Set with the <code>jruby.debug.loadService</code> system property.
      */
     public static final boolean DEBUG_LOAD_SERVICE = Options.DEBUG_LOADSERVICE.load();
 
     /**
      * Turn on timings of the load service (requires and loads).
      *
-     * Set with the <tt>jruby.debug.loadService.timing</tt> system property.
+     * Set with the <code>jruby.debug.loadService.timing</code> system property.
      */
     public static final boolean DEBUG_LOAD_TIMINGS = Options.DEBUG_LOADSERVICE_TIMING.load();
 
     /**
      * Turn on debugging of subprocess launching.
      *
-     * Set with the <tt>jruby.debug.launch</tt> system property.
+     * Set with the <code>jruby.debug.launch</code> system property.
      */
     public static final boolean DEBUG_LAUNCHING = Options.DEBUG_LAUNCH.load();
 
     /**
      * Turn on debugging of script resolution with "-S".
      *
-     * Set with the <tt>jruby.debug.scriptResolution</tt> system property.
+     * Set with the <code>jruby.debug.scriptResolution</code> system property.
      */
     public static final boolean DEBUG_SCRIPT_RESOLUTION = Options.DEBUG_SCRIPTRESOLUTION.load();
 
@@ -1771,7 +1788,7 @@ public class RubyInstanceConfig {
      * Whether to calculate consistent hashes across JVM instances, or to ensure
      * un-predicatable hash values using SecureRandom.
      *
-     * Set with the <tt>jruby.consistent.hashing.enabled</tt> system property.
+     * Set with the <code>jruby.consistent.hashing.enabled</code> system property.
      */
     public static final boolean CONSISTENT_HASHING_ENABLED = Options.CONSISTENT_HASHING.load();
 
@@ -1783,23 +1800,40 @@ public class RubyInstanceConfig {
 
     private static int initJavaBytecodeVersion() {
         final String specVersion = Options.BYTECODE_VERSION.load();
-        switch ( specVersion ) {
-            case "1.6" :
-                return Opcodes.V1_6;
-            case "1.7" :
-                return Opcodes.V1_7;
-            case "1.8" : case "8" : default :
+        if (specVersion.indexOf('.') != -1) {
+            switch (specVersion) {
+                default:
+                    System.err.println("unsupported Java version, using 1.8: " + specVersion);
+                case "1.8":
+                    return Opcodes.V1_8;
+            }
+        }
+
+        int version = Integer.parseInt(specVersion);
+        switch (version) {
+            case 8 :
                 return Opcodes.V1_8; // 52
-            case "9" :
+            case 9 :
                 return Opcodes.V9;
-            case "10" :
+            case 10 :
                 return Opcodes.V10;
-            case "11" :
+            case 11 :
                 return Opcodes.V11;
-            case "12" :
+            case 12 :
                 return Opcodes.V12;
-            case "13" :
+            case 13 :
                 return Opcodes.V13;
+            case 14 :
+                return Opcodes.V14;
+            case 15 :
+                return Opcodes.V15;
+            case 16 :
+                return Opcodes.V16;
+            case 17 :
+                return Opcodes.V17;
+            case 18 :
+            default :
+                return Opcodes.V18;
         }
     }
 
@@ -2016,4 +2050,7 @@ public class RubyInstanceConfig {
 
     @Deprecated
     public static final boolean USE_GENERATED_HANDLES = false;
+
+    @Deprecated
+    public static final boolean FASTOPS_COMPILE_ENABLED = Options.COMPILE_FASTOPS.load();
 }
