@@ -42,6 +42,7 @@ import org.jcodings.specific.UTF8Encoding;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.ast.util.ArgsUtil;
+import org.jruby.runtime.Arity;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.CallSite;
 import org.jruby.runtime.ClassIndex;
@@ -62,6 +63,7 @@ import static org.jruby.RubyEnumerator.SizeFn;
 import static org.jruby.RubyEnumerator.enumeratorizeWithSize;
 import static org.jruby.util.Numeric.f_gcd;
 import static org.jruby.util.Numeric.f_lcm;
+import static org.jruby.util.Numeric.f_zero_p;
 
 /** Implementation of the Integer class.
  *
@@ -70,6 +72,9 @@ import static org.jruby.util.Numeric.f_lcm;
 @JRubyClass(name="Integer", parent="Numeric", overrides = {RubyFixnum.class, RubyBignum.class})
 public abstract class RubyInteger extends RubyNumeric {
 
+    private static final int BIT_SIZE = 64;
+    private static final long MAX = (1L << (BIT_SIZE - 1)) - 1;
+
     public static RubyClass createIntegerClass(Ruby runtime) {
         RubyClass integer = runtime.defineClass("Integer", runtime.getNumeric(),
                 ObjectAllocator.NOT_ALLOCATABLE_ALLOCATOR);
@@ -153,6 +158,11 @@ public abstract class RubyInteger extends RubyNumeric {
         return num.convertToInteger().sqrt(context);
     }
 
+    @JRubyMethod(meta = true)
+    public static IRubyObject try_convert(ThreadContext context, IRubyObject self, IRubyObject num) {
+        return TypeConverter.checkIntegerType(context, num);
+    }
+
     public abstract IRubyObject sqrt(ThreadContext context);
 
     // floorSqrt :: unsigned long -> unsigned long
@@ -225,6 +235,7 @@ public abstract class RubyInteger extends RubyNumeric {
             long i;
             for (i = from; i < to; i++) {
                 block.yield(context, nil);
+                context.pollThreadEvents();
             }
             if (i <= to) {
                 block.yield(context, nil);
@@ -234,6 +245,7 @@ public abstract class RubyInteger extends RubyNumeric {
             long i;
             for (i = from; i < to; i++) {
                 block.yield(context, RubyFixnum.newFixnum(runtime, i));
+                context.pollThreadEvents();
             }
             if (i <= to) {
                 block.yield(context, RubyFixnum.newFixnum(runtime, i));
@@ -330,11 +342,11 @@ public abstract class RubyInteger extends RubyNumeric {
             IRubyObject i = RubyFixnum.zero(runtime);
             RubyFixnum one = RubyFixnum.one(runtime);
             while (true) {
-                if (!sites(context).op_lt.call(context, i, i, this).isTrue()) {
+                if (!((RubyInteger) i).op_lt(context, this).isTrue()) {
                     break;
                 }
                 block.yield(context, i);
-                i = sites(context).op_plus.call(context, i, i, one);
+                i = ((RubyInteger) i).op_plus(context, one);
             }
             return this;
         } else {
@@ -371,11 +383,20 @@ public abstract class RubyInteger extends RubyNumeric {
         }
     }
 
+    static final byte[][] SINGLE_CHAR_BYTES;
+    static {
+        SINGLE_CHAR_BYTES = new byte[256][];
+        for (int i = 0; i < 256; i++) {
+            byte[] bytes = new byte[] { (byte) i };
+            SINGLE_CHAR_BYTES[i] = bytes;
+        }
+    }
+
     static final ByteList[] SINGLE_CHAR_BYTELISTS;
     static {
         SINGLE_CHAR_BYTELISTS = new ByteList[256];
         for (int i = 0; i < 256; i++) {
-            ByteList bytes = new ByteList(new byte[] { (byte) i }, false);
+            ByteList bytes = new ByteList(SINGLE_CHAR_BYTES[i], false);
             SINGLE_CHAR_BYTELISTS[i] = bytes;
             bytes.setEncoding(i < 0x80 ? USASCIIEncoding.INSTANCE : ASCIIEncoding.INSTANCE);
         }
@@ -391,7 +412,7 @@ public abstract class RubyInteger extends RubyNumeric {
     static {
         SINGLE_CHAR_UTF8_BYTELISTS = new ByteList[128];
         for (int i = 0; i < 128; i++) {
-            ByteList bytes = new ByteList(new byte[] { (byte) i }, false);
+            ByteList bytes = new ByteList(SINGLE_CHAR_BYTES[i], false);
             SINGLE_CHAR_UTF8_BYTELISTS[i] = bytes;
             bytes.setEncoding(UTF8Encoding.INSTANCE);
         }
@@ -422,10 +443,10 @@ public abstract class RubyInteger extends RubyNumeric {
         ByteList bytes;
         if (enc == USASCIIEncoding.INSTANCE) {
             bytes = singleCharByteList(b);
-        } else if ((b & 0xFF) < 0x80 && enc == RubyString.UTF8) {
+        } else if (Byte.toUnsignedInt(b) < 0x80 && enc == RubyString.UTF8) {
             bytes = singleCharUTF8ByteList(b);
         } else {
-            return new RubyString(runtime, meta, new ByteList(new byte[]{b}, enc));
+            return RubyString.newStringShared(runtime, SINGLE_CHAR_BYTES[Byte.toUnsignedInt(b)], enc);
         }
 
         // use shared for cached bytelists
@@ -439,21 +460,58 @@ public abstract class RubyInteger extends RubyNumeric {
     public RubyString chr(ThreadContext context) {
         Ruby runtime = context.runtime;
 
-        // rb_num_to_uint
-        long i = getLongValue() & 0xFFFFFFFFL;
-        int c = (int) i;
+        long uint = toUnsignedInteger(runtime);
 
-        Encoding enc;
-
-        if (i > 0xff) {
-            enc = runtime.getDefaultInternalEncoding();
+        if (uint > 0xff) {
+            Encoding enc = runtime.getDefaultInternalEncoding();
             if (enc == null) {
-                throw runtime.newRangeError(toString() + " out of char range");
+                throw runtime.newRangeError(uint + " out of char range");
             }
-            return chrCommon(context, c, enc);
+            return chrCommon(context, uint, enc);
         }
 
-        return RubyString.newStringShared(runtime, SINGLE_CHAR_BYTELISTS[c]);
+        return RubyString.newStringShared(runtime, SINGLE_CHAR_BYTELISTS[(int) uint]);
+    }
+
+    private long toUnsignedInteger(Ruby runtime) {
+        // rb_num_to_uint
+        long uintResult = numToUint(this);
+        long uint = uintResult >>> 32;
+        int ret = (int) (uintResult & 0xFFFFFFFF);
+        if (ret == 0) {
+        } else if (this instanceof RubyFixnum) {
+            throw runtime.newRangeError(getLongValue() + " out of char range");
+        } else {
+            throw runtime.newRangeError("bignum out of char range");
+        }
+        return uint;
+    }
+
+    public static final int NUMERR_TYPE = 1;
+    public static final int NUMERR_NEGATIVE = 2;
+    public static final int NUMERR_TOOLARGE = 3;
+
+    /**
+     * Simulate CRuby's rb_num_to_uint by returning a single long; the top 4 bytes will be the uint and the bottom
+     * four bytes will be the result code. See {@link #NUMERR_TYPE}, {@link #NUMERR_NEGATIVE}, and {@link #NUMERR_TOOLARGE}.
+     *
+     * @param val the object to convert to a uint
+     * @return the value and result code, with the top four bytes being the result code (zero if no error)
+     */
+    public static long numToUint(IRubyObject val) {
+        if (val instanceof RubyFixnum) {
+            long v = fix2long(val);
+            if (v > 0xFFFFFFFFL) return NUMERR_TOOLARGE;
+            if (v < 0) return NUMERR_NEGATIVE;
+            return v << 32;
+        }
+
+        if (val instanceof RubyBignum) {
+            if (((RubyBignum) val).isNegative()) return NUMERR_NEGATIVE;
+            /* long is 64bit */
+            return NUMERR_TOOLARGE;
+        }
+        return NUMERR_TYPE;
     }
 
     @Deprecated
@@ -465,8 +523,7 @@ public abstract class RubyInteger extends RubyNumeric {
     public RubyString chr(ThreadContext context, IRubyObject arg) {
         Ruby runtime = context.runtime;
 
-        // rb_num_to_uint
-        long i = getLongValue() & 0xFFFFFFFFL;
+        long uint = toUnsignedInteger(runtime);
 
         Encoding enc;
         if (arg instanceof RubyEncoding) {
@@ -474,7 +531,7 @@ public abstract class RubyInteger extends RubyNumeric {
         } else {
             enc =  arg.convertToString().toEncoding(runtime);
         }
-        return chrCommon(context, i, enc);
+        return chrCommon(context, uint, enc);
     }
 
     @Deprecated
@@ -512,7 +569,7 @@ public abstract class RubyInteger extends RubyNumeric {
         return this;
     }
 
-    @JRubyMethod(name = "ceil", required = 1)
+    @JRubyMethod(name = "ceil")
     public abstract IRubyObject ceil(ThreadContext context, IRubyObject arg);
 
     @JRubyMethod(name = "floor")
@@ -520,7 +577,7 @@ public abstract class RubyInteger extends RubyNumeric {
         return this;
     }
 
-    @JRubyMethod(name = "floor", required = 1)
+    @JRubyMethod(name = "floor")
     public abstract IRubyObject floor(ThreadContext context, IRubyObject arg);
 
     @JRubyMethod(name = "truncate")
@@ -528,7 +585,7 @@ public abstract class RubyInteger extends RubyNumeric {
         return this;
     }
 
-    @JRubyMethod(name = "truncate", required = 1)
+    @JRubyMethod(name = "truncate")
     public abstract IRubyObject truncate(ThreadContext context, IRubyObject arg);
 
     @Override
@@ -667,8 +724,10 @@ public abstract class RubyInteger extends RubyNumeric {
     /** integer_rationalize
      *
      */
-    @JRubyMethod(name = "rationalize", optional = 1)
+    @JRubyMethod(name = "rationalize", optional = 1, checkArity = false)
     public IRubyObject rationalize(ThreadContext context, IRubyObject[] args) {
+        Arity.checkArgumentCount(context, args, 0, 1);
+
         return to_r(context);
     }
 
@@ -689,19 +748,19 @@ public abstract class RubyInteger extends RubyNumeric {
 
     @JRubyMethod(name = "allbits?")
     public IRubyObject allbits_p(ThreadContext context, IRubyObject other) {
-        IRubyObject mask = TypeConverter.checkIntegerType(context, other);
+        IRubyObject mask = TypeConverter.checkToInteger(context, other);
         return ((RubyInteger) op_and(context, mask)).op_equal(context, mask);
     }
 
     @JRubyMethod(name = "anybits?")
     public IRubyObject anybits_p(ThreadContext context, IRubyObject other) {
-        IRubyObject mask = TypeConverter.checkIntegerType(context, other);
+        IRubyObject mask = TypeConverter.checkToInteger(context, other);
         return ((RubyInteger) op_and(context, mask)).zero_p(context).isTrue() ? context.fals : context.tru;
     }
 
     @JRubyMethod(name = "nobits?")
     public IRubyObject nobits_p(ThreadContext context, IRubyObject other) {
-        IRubyObject mask = TypeConverter.checkIntegerType(context, other);
+        IRubyObject mask = TypeConverter.checkToInteger(context, other);
         return ((RubyInteger) op_and(context, mask)).zero_p(context);
     }
 
@@ -722,7 +781,15 @@ public abstract class RubyInteger extends RubyNumeric {
     @Override
     @JRubyMethod(name = "fdiv")
     public IRubyObject fdiv(ThreadContext context, IRubyObject y) {
-        return fdivDouble(context, y);
+        RubyInteger x = this;
+        if (y instanceof RubyInteger && !f_zero_p(context, y)) {
+            IRubyObject gcd = gcd(context, y);
+            if (!f_zero_p(context, gcd)) {
+                x = (RubyInteger)x.idiv(context, gcd);
+                y = ((RubyInteger)y).idiv(context, gcd);
+            }
+        }
+        return x.fdivDouble(context, y);
     }
 
     public abstract IRubyObject fdivDouble(ThreadContext context, IRubyObject y);
@@ -867,6 +934,7 @@ public abstract class RubyInteger extends RubyNumeric {
 
         if (m instanceof RubyFixnum) {
             long mm = ((RubyFixnum) m).value;
+            if (mm == 1) return RubyFixnum.zero(runtime);
             RubyFixnum modulo = (RubyFixnum) modulo(context, m);
             if (mm <= HALF_LONG_MSB) {
                 return modulo.intPowTmp1(context, (RubyInteger) b, mm, negaFlg);
@@ -875,6 +943,7 @@ public abstract class RubyInteger extends RubyNumeric {
             }
         }
         if (m instanceof RubyBignum) {
+            if (((RubyBignum) m).value == BigInteger.ONE) return RubyFixnum.zero(runtime);
             return ((RubyInteger) modulo(context, m)).intPowTmp3(context, (RubyInteger) b, (RubyBignum) m, negaFlg);
         }
         // not reached
@@ -925,7 +994,75 @@ public abstract class RubyInteger extends RubyNumeric {
     public abstract IRubyObject op_xor(ThreadContext context, IRubyObject other);
 
     @JRubyMethod(name = "[]")
-    public abstract IRubyObject op_aref(ThreadContext context, IRubyObject other);
+    public IRubyObject op_aref(ThreadContext context, IRubyObject index) {
+        if (index instanceof RubyRange) {
+            RubyRange range = (RubyRange) index;
+            IRubyObject beg = range.begin(context);
+            IRubyObject end = range.end(context);
+            boolean isExclusive = range.isExcludeEnd();
+
+            if (!end.isNil()) end = end.convertToInteger();
+
+            if (beg.isNil()) {
+                if (!negativeInt(context, end)) {
+                    if (!isExclusive) {
+                        end = ((RubyInteger) end).op_plus(context, context.runtime.newFixnum(1));
+                    }
+
+                    RubyInteger mask = generateMask(context, end);
+                    if (((RubyInteger) op_and(context, mask)).isZero()) {
+                        return context.runtime.newFixnum(0);
+                    } else {
+                        throw context.runtime.newArgumentError("The beginless range for Integer#[] results in infinity");
+                    }
+                } else {
+                    return context.runtime.newFixnum(0);
+                }
+            }
+            beg = beg.convertToInteger();
+            IRubyObject num = op_rshift(context, beg);
+            long cmp = compareIndexes(context, beg, end);
+            if (!end.isNil() && cmp < 0) {
+                IRubyObject length = ((RubyInteger) end).op_minus(context, beg);
+                if (!isExclusive) {
+                    length = ((RubyInteger) length).op_plus(context, context.runtime.newFixnum(1));
+                }
+                RubyInteger mask = generateMask(context, length);
+                num = (((RubyInteger) num).op_and(context, mask));
+                return num;
+            } else if (cmp == 0) {
+                if (isExclusive) return context.runtime.newFixnum(0);
+                index = beg;
+            } else {
+                return num;
+            }
+        }
+
+        return op_aref_subclass(context, index);
+    }
+
+    private static long compareIndexes(ThreadContext context, IRubyObject beg, IRubyObject end) {
+        IRubyObject r = beg.callMethod(context, "<=>", end);
+
+        if (r.isNil()) return MAX;
+
+        JavaSites.IntegerSites sites = sites(context);
+        return RubyComparable.cmpint(context, sites.op_gt, sites.op_lt, r, beg, end);
+    }
+
+    protected abstract IRubyObject op_aref_subclass(ThreadContext context, IRubyObject index);
+
+    @JRubyMethod(name = "[]")
+    public IRubyObject op_aref(ThreadContext context, IRubyObject index, IRubyObject length) {
+        IRubyObject num = op_rshift(context, index);
+        IRubyObject mask = generateMask(context, length);
+        return ((RubyInteger) num).op_and(context, mask);
+    }
+
+    RubyInteger generateMask(ThreadContext context, IRubyObject length) {
+        RubyFixnum one = context.runtime.newFixnum(1);
+        return (RubyInteger) ((RubyInteger) one.op_lshift(context, length)).op_minus(context, one);
+    }
 
     @JRubyMethod(name = "<<")
     public abstract IRubyObject op_lshift(ThreadContext context, IRubyObject other);
