@@ -87,7 +87,7 @@ public class RubyMath {
         return (y < 0.0) ? -abs : abs;
     }
 
-    @JRubyMethod(name = "atan2", required = 2, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "atan2", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat atan2(ThreadContext context, IRubyObject recv, IRubyObject x, IRubyObject y) {
         double valuea = RubyNumeric.num2dbl(context, x);
         double valueb = RubyNumeric.num2dbl(context, y);
@@ -100,7 +100,7 @@ public class RubyMath {
         return atan2(context, recv, x, y);
     }
 
-    @JRubyMethod(name = "cos", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "cos", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat cos(ThreadContext context, IRubyObject recv, IRubyObject x) {
         return RubyFloat.newFloat(context.runtime, Math.cos(RubyNumeric.num2dbl(context, x)));
     }
@@ -110,7 +110,7 @@ public class RubyMath {
         return cos(context, recv, x);
     }
 
-    @JRubyMethod(name = "sin", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "sin", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat sin(ThreadContext context, IRubyObject recv, IRubyObject x) {
         return RubyFloat.newFloat(context.runtime, Math.sin(RubyNumeric.num2dbl(context, x)));
     }
@@ -120,7 +120,7 @@ public class RubyMath {
         return sin(context, recv, x);
     }
 
-    @JRubyMethod(name = "tan", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "tan", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat tan(ThreadContext context, IRubyObject recv,  IRubyObject x) {
         return RubyFloat.newFloat(context.runtime, Math.tan(RubyNumeric.num2dbl(context, x)));
     }
@@ -130,7 +130,7 @@ public class RubyMath {
         return tan(context, recv, x);
     }
 
-    @JRubyMethod(name = "asin", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "asin", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat asin(ThreadContext context, IRubyObject recv, IRubyObject x) {
         double value = RubyNumeric.num2dbl(context, x);
 
@@ -144,7 +144,7 @@ public class RubyMath {
         return asin(context, recv, x);
     }
 
-    @JRubyMethod(name = "acos", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "acos", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat acos(ThreadContext context, IRubyObject recv, IRubyObject x) {
         double value = RubyNumeric.num2dbl(context, x);
 
@@ -158,7 +158,7 @@ public class RubyMath {
         return acos(context, recv, x);
     }
 
-    @JRubyMethod(name = "atan", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "atan", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat atan(ThreadContext context, IRubyObject recv, IRubyObject x) {
         return RubyFloat.newFloat(context.runtime, Math.atan(RubyNumeric.num2dbl(context, x)));
     }
@@ -168,7 +168,7 @@ public class RubyMath {
         return atan(context, recv, x);
     }
 
-    @JRubyMethod(name = "cosh", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "cosh", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat cosh(ThreadContext context, IRubyObject recv, IRubyObject x) {
         double value = RubyNumeric.num2dbl(context, x);
         
@@ -180,7 +180,7 @@ public class RubyMath {
         return cosh(context, recv, x);
     }
 
-    @JRubyMethod(name = "sinh", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "sinh", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat sinh(ThreadContext context, IRubyObject recv, IRubyObject x) {
         double value = RubyNumeric.num2dbl(context, x);
         
@@ -192,7 +192,7 @@ public class RubyMath {
         return sinh(context, recv, x);
     }
 
-    @JRubyMethod(name = "tanh", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "tanh", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat tanh(ThreadContext context, IRubyObject recv, IRubyObject x) {
         return RubyFloat.newFloat(context.runtime, Math.tanh(RubyNumeric.num2dbl(context, x)));
     }
@@ -202,7 +202,7 @@ public class RubyMath {
         return tanh(context, recv, x);
     }
 
-    @JRubyMethod(name = "acosh", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "acosh", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat acosh(ThreadContext context, IRubyObject recv, IRubyObject x) {
         double value = RubyNumeric.num2dbl(context, x);
         double result;
@@ -249,7 +249,7 @@ public class RubyMath {
         .34904658524827565638313923706880e-17
     };
 
-    @JRubyMethod(name = "asinh", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "asinh", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat asinh(ThreadContext context, IRubyObject recv, IRubyObject x) {
         double value = RubyNumeric.num2dbl(context, x);
         double  y = Math.abs(value);
@@ -295,7 +295,7 @@ public class RubyMath {
         .4497954546494931083083327624533e-18
     };
 
-    @JRubyMethod(name = "atanh", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "atanh", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat atanh(ThreadContext context, IRubyObject recv, IRubyObject x) {
         double value = RubyNumeric.num2dbl(context, x);
 
@@ -326,7 +326,7 @@ public class RubyMath {
         return atanh(context, recv, x);
     }
 
-    @JRubyMethod(name = "exp", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "exp", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat exp(ThreadContext context, IRubyObject recv, IRubyObject exponent) {
         return exp(context, exponent);
     }
@@ -340,12 +340,28 @@ public class RubyMath {
         return exp(context, recv, exponent);
     }
 
-    private static RubyFloat logCommon(ThreadContext context, double value, double base, String msg) {
-        if (value < 0 || base < 0) throw context.runtime.newMathDomainError(msg);
+    // MRI : get_double_rshift
+    private static double[] get_double_rshift(ThreadContext context, IRubyObject x) {
+        int numbits = 0;
+        if (x instanceof RubyBignum) {
+            RubyBignum bignum = (RubyBignum)x;
+            numbits = bignum.getValue().abs().bitLength();
+            if (bignum.getValue().signum() > 0 && numbits >= DBL_MAX_EXP) {
+                numbits -= DBL_MANT_DIG;
+                x = bignum.op_rshift(context, numbits);
+            } else {
+                numbits = 0;
+            }
+        }
 
-        return RubyFloat.newFloat(context.runtime, Math.log(value) / Math.log(base));
+        return new double[]{RubyNumeric.num2dbl(context, x), numbits};
     }
 
+    private static int DBL_MANT_DIG = 53;
+    private static int DBL_MAX_EXP = 1024;
+    private static double LOG_E_2 = Math.log(2);
+    private static double LOG_10_2 = Math.log10(2);
+
     /** Returns the natural logarithm of x.
      * 
      */
@@ -355,15 +371,27 @@ public class RubyMath {
     }
 
     public static RubyFloat log(ThreadContext context, IRubyObject val) {
-        double value = RubyNumeric.num2dbl(context, val);
-        return logCommon(context, value, Math.E, "log");
+        double [] ret = get_double_rshift(context, val);
+
+        if (ret[0] < 0) {
+            throw context.runtime.newMathDomainError("log");
+        }
+
+        /* log(d * 2 ** numbits) */
+        return RubyFloat.newFloat(context.runtime, Math.log(ret[0]) + ret[1] * LOG_E_2);
     }
 
     @JRubyMethod(name = "log", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat log(ThreadContext context, IRubyObject recv, IRubyObject val, IRubyObject base) {
-        double value = RubyNumeric.num2dbl(context, val);
+        double [] ret = get_double_rshift(context, val);
         double _base = RubyNumeric.num2dbl(context, base);
-        return logCommon(context, value, _base, "log");
+
+        if (ret[0] < 0 || _base < 0) {
+            throw context.runtime.newMathDomainError("log");
+        }
+
+        /* log(d * 2 ** numbits) / log(base) */
+        return RubyFloat.newFloat(context.runtime, Math.log(ret[0]) / Math.log(_base) + ret[1]);
     }
 
     public static RubyFloat log(ThreadContext context, IRubyObject recv, IRubyObject... args) {
@@ -381,15 +409,16 @@ public class RubyMath {
     /** Returns the base 10 logarithm of x.
      * 
      */
-    @JRubyMethod(name = "log10", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "log10", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat log10(ThreadContext context, IRubyObject recv, IRubyObject x) {
-        double value = RubyNumeric.num2dbl(context, x);
+        double [] ret = get_double_rshift(context, x);
 
-        if (value < 0) {
+        if (ret[0] < 0) {
             throw context.runtime.newMathDomainError("log10");
         }
 
-        return RubyFloat.newFloat(context.runtime, Math.log10(value));
+        /* log10(d * 2 ** numbits) */
+        return RubyFloat.newFloat(context.runtime, Math.log10(ret[0]) + ret[1] * LOG_10_2);
     }
 
     @Deprecated
@@ -400,9 +429,16 @@ public class RubyMath {
     /** Returns the base 2 logarithm of x.
      *
      */
-    @JRubyMethod(name = "log2", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "log2", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat log2(ThreadContext context, IRubyObject recv, IRubyObject x) {
-        return logCommon(context, RubyNumeric.num2dbl(context, x), 2, "log2");
+        double [] ret = get_double_rshift(context, x);
+
+        if (ret[0] < 0) {
+            throw context.runtime.newMathDomainError("log2");
+        }
+
+        /* log2(d * 2 ** numbits) */
+        return RubyFloat.newFloat(context.runtime, Math.log(ret[0]) / LOG_E_2 + ret[1]);
     }
 
     @Deprecated
@@ -410,7 +446,7 @@ public class RubyMath {
         return log2(context, recv, x);
     }
 
-    @JRubyMethod(name = "sqrt", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "sqrt", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat sqrt(ThreadContext context, IRubyObject recv, IRubyObject x) {
         double value = RubyNumeric.num2dbl(context, x);
 
@@ -424,11 +460,9 @@ public class RubyMath {
         return sqrt(context, recv, x);
     }
 
-    @JRubyMethod(name = "cbrt", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "cbrt", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat cbrt(ThreadContext context, IRubyObject recv, IRubyObject x) {
         double result = Math.cbrt(RubyNumeric.num2dbl(context, x));
-
-        domainCheck(recv, result, "cbrt");
         
         return RubyFloat.newFloat(context.runtime, result);
     }
@@ -437,7 +471,7 @@ public class RubyMath {
         return hypot19(context, recv, x, y);
     }    
 
-    @JRubyMethod(name = "hypot", required = 2, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "hypot", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat hypot19(ThreadContext context, IRubyObject recv, IRubyObject x, IRubyObject y) {
         double valuea = RubyNumeric.num2dbl(context, x);
         double valueb = RubyNumeric.num2dbl(context, y);
@@ -465,7 +499,7 @@ public class RubyMath {
      * Where mantissa is in the range of [.5, 1)
      *
      */
-    @JRubyMethod(name = "frexp", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "frexp", module = true, visibility = Visibility.PRIVATE)
     public static RubyArray frexp(ThreadContext context, IRubyObject recv, IRubyObject other) {
         double mantissa = RubyNumeric.num2dbl(context, other);
         short sign = 1;
@@ -498,7 +532,7 @@ public class RubyMath {
     /*
      * r = x * 2 ** y
      */
-    @JRubyMethod(name = "ldexp", required = 2, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "ldexp", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat ldexp(ThreadContext context, IRubyObject recv, IRubyObject mantissa, IRubyObject exponent) {
         double m = RubyNumeric.num2dbl(context, mantissa);
         int e = RubyNumeric.num2int(exponent);
@@ -532,7 +566,7 @@ public class RubyMath {
          -.126124551191552258324954248533e-18
     };
 
-    @JRubyMethod(name = "erf", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "erf", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat erf(ThreadContext context, IRubyObject recv, IRubyObject x) {
         double value = RubyNumeric.num2dbl(context, x);
 
@@ -620,7 +654,7 @@ public class RubyMath {
          .194744338223207851429197867821e-18
     };
 
-    @JRubyMethod(name = "erfc", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "erfc", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat erfc(ThreadContext context, IRubyObject recv, IRubyObject x) {
         double value = RubyNumeric.num2dbl(context, x);
         double  result;
@@ -713,7 +747,7 @@ public class RubyMath {
      * New asymptotic expansion for the &Gamma;(x) function</a>
      */
 
-    @JRubyMethod(name = "gamma", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "gamma", module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat gamma(ThreadContext context, IRubyObject recv, IRubyObject x) {
         double value = RubyKernel.new_float(context.runtime, x).value;
         double result = nemes_gamma(value);
@@ -752,7 +786,7 @@ public class RubyMath {
      * @see #gamma(ThreadContext, org.jruby.runtime.builtin.IRubyObject, org.jruby.runtime.builtin.IRubyObject)
      */
 
-    @JRubyMethod(name = "lgamma", required = 1, module = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "lgamma", module = true, visibility = Visibility.PRIVATE)
     public static RubyArray lgamma(ThreadContext context, IRubyObject recv, IRubyObject x) {
         double value = RubyKernel.new_float(context.runtime, x).value;
         // JRUBY-4653: Could this error checking done more elegantly?
