@@ -45,6 +45,7 @@ import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.api.API;
 import org.jruby.exceptions.JumpException;
+import org.jruby.runtime.Arity;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.BlockCallback;
 import org.jruby.runtime.CallBlock;
@@ -71,12 +72,11 @@ import org.jruby.runtime.component.VariableEntry;
 import org.jruby.runtime.invokedynamic.MethodNames;
 import org.jruby.runtime.marshal.MarshalStream;
 import org.jruby.runtime.marshal.UnmarshalStream;
+import org.jruby.util.Numeric;
 import org.jruby.util.TypeConverter;
 
 import static org.jruby.RubyEnumerator.SizeFn;
-import static org.jruby.RubyEnumerator.enumeratorizeWithSize;
 import static org.jruby.RubyNumeric.intervalStepSize;
-import static org.jruby.runtime.Helpers.invokedynamic;
 
 import static org.jruby.runtime.Visibility.PRIVATE;
 
@@ -88,6 +88,7 @@ public class RubyRange extends RubyObject {
 
     private IRubyObject begin;
     private IRubyObject end;
+    private boolean isBeginless;
     private boolean isExclusive;
     private boolean isEndless;
     private boolean isInited = false;
@@ -143,7 +144,7 @@ public class RubyRange extends RubyObject {
     }
 
     final boolean checkBegin(long length) {
-        long beg = RubyNumeric.num2long(this.begin);
+        long beg = isBeginless ? 0 : RubyNumeric.num2long(this.begin);
         if (beg < 0) {
             beg += length;
             if (beg < 0) {
@@ -156,7 +157,7 @@ public class RubyRange extends RubyObject {
     }
 
     final long[] begLen(long len, int err) {
-        long beg = RubyNumeric.num2long(this.begin);
+        long beg = isBeginless ? 0 : RubyNumeric.num2long(this.begin);
         long end = isEndless ? -1: RubyNumeric.num2long(this.end);
 
         if (beg < 0) {
@@ -196,12 +197,12 @@ public class RubyRange extends RubyObject {
     }
 
     final long begLen0(long len) {
-        long beg = RubyNumeric.num2long(this.begin);
+        long beg = isBeginless ? 0 : RubyNumeric.num2long(this.begin);
 
         if (beg < 0) {
             beg += len;
             if (beg < 0) {
-                throw getRuntime().newRangeError(beg + ".." + (isExclusive ? "." : "") + end + " out of range");
+                throw getRuntime().newRangeError((beg - len) + ".." + (isExclusive ? "." : "") + end + " out of range");
             }
         }
 
@@ -225,15 +226,16 @@ public class RubyRange extends RubyObject {
         return len;
     }
 
+    // MRI: rb_range_component_beg_len
     final int[] begLenInt(int len, final int err) {
-        int beg = RubyNumeric.num2int(this.begin);
+        int beg = isBeginless ? 0 : RubyNumeric.num2int(this.begin);
         int end = isEndless ? -1 : RubyNumeric.num2int(this.end);
 
         if (beg < 0) {
             beg += len;
             if (beg < 0) {
                 if (err != 0) {
-                    throw getRuntime().newRangeError(beg + ".." + (isExclusive ? "." : "") + end + " out of range");
+                    throw getRuntime().newRangeError(this.begin + ".." + (isExclusive ? "." : "") + this.end + " out of range");
                 }
                 return null;
             }
@@ -242,7 +244,7 @@ public class RubyRange extends RubyObject {
         if (err == 0 || err == 2) {
             if (beg > len) {
                 if (err != 0) {
-                    throw getRuntime().newRangeError(beg + ".." + (isExclusive ? "." : "") + end + " out of range");
+                    throw getRuntime().newRangeError(this.begin + ".." + (isExclusive ? "." : "") + this.end + " out of range");
                 }
                 return null;
             }
@@ -266,7 +268,7 @@ public class RubyRange extends RubyObject {
     }
 
     private void init(ThreadContext context, IRubyObject begin, IRubyObject end, boolean isExclusive) {
-        if (!(begin instanceof RubyFixnum && end instanceof RubyFixnum) && !end.isNil()) {
+        if (!(begin instanceof RubyFixnum && end instanceof RubyFixnum) && !end.isNil() && !begin.isNil()) {
             IRubyObject result = invokedynamic(context, begin, MethodNames.OP_CMP, end);
             if (result.isNil()) {
                 throw context.runtime.newArgumentError("bad value for range");
@@ -277,28 +279,34 @@ public class RubyRange extends RubyObject {
         this.end = end;
         this.isExclusive = isExclusive;
         this.isEndless = end.isNil();
+        this.isBeginless = begin.isNil();
         this.isInited = true;
+        if (metaClass.getClassIndex() == ClassIndex.RANGE) {
+            this.setFrozen(true);
+        }
     }
 
-    @JRubyMethod(required = 2, optional = 1, visibility = PRIVATE)
+    @JRubyMethod(required = 2, optional = 1, checkArity = false, visibility = PRIVATE)
     public IRubyObject initialize(ThreadContext context, IRubyObject[] args, Block unusedBlock) {
-        if (this.isInited) {
-            throw context.runtime.newNameError("`initialize' called twice", context.runtime.newSymbol("initialize"));
-        }
+        Arity.checkArgumentCount(context, args, 2, 3);
+
+        if (this.isInited) throw context.runtime.newFrozenError("`initialize' called twice", this);
         checkFrozen();
         init(context, args[0], args[1], args.length > 2 && args[2].isTrue());
         this.isInited = true;
         return context.nil;
     }
 
-    @JRubyMethod(required = 1, visibility = PRIVATE)
+    @JRubyMethod(visibility = PRIVATE)
     public IRubyObject initialize_copy(ThreadContext context, IRubyObject original) {
-        if (this.isInited) {
-            throw context.runtime.newNameError("`initialize' called twice", context.runtime.newSymbol("initialize"));
-        }
+        if (this.isInited) throw context.runtime.newFrozenError("`initialize' called twice", this);
 
         RubyRange other = (RubyRange) original;
-        init(context, other.begin, other.end, other.isExclusive);
+        this.begin = other.begin;
+        this.end = other.end;
+        this.isExclusive = other.isExclusive;
+        this.isEndless = other.end.isNil();
+        this.isBeginless = other.begin.isNil();
         this.isInited = true;
         return context.nil;
     }
@@ -328,20 +336,16 @@ public class RubyRange extends RubyObject {
     }
 
     private static RubyString inspectValue(final ThreadContext context, IRubyObject value) {
-        return (RubyString) context.safeRecurse(INSPECT_RECURSIVE, value, value, "inspect", true);
+        return (RubyString) context.safeRecurse(RubyRange::inspectValueRecursive, value, value, "inspect", true);
     }
 
-    private static class InspectRecursive implements ThreadContext.RecursiveFunctionEx<IRubyObject> {
-        @Override
-        public IRubyObject call(ThreadContext context, IRubyObject state, IRubyObject obj, boolean recur) {
-            if (recur) {
-                return RubyString.newString(context.runtime, ((RubyRange) obj).isExclusive ? "(... ... ...)" : "(... .. ...)");
-            } else {
-                return inspect(context, obj);
-            }
+    private static IRubyObject inspectValueRecursive(ThreadContext context, IRubyObject state, IRubyObject obj, boolean recur) {
+        if (recur) {
+            return RubyString.newString(context.runtime, ((RubyRange) obj).isExclusive ? "(... ... ...)" : "(... .. ...)");
+        } else {
+            return inspect(context, obj);
         }
     }
-    private static final InspectRecursive INSPECT_RECURSIVE = new InspectRecursive();
 
     private static final byte[] DOTDOTDOT = new byte[]{'.', '.', '.'};
 
@@ -352,12 +356,10 @@ public class RubyRange extends RubyObject {
 
     @JRubyMethod(name = "inspect")
     public RubyString inspect(final ThreadContext context) {
-        RubyString i1 = inspectValue(context, begin).strDup(context.runtime);
-        RubyString i2 = isEndless ? RubyString.newEmptyString(context.runtime): inspectValue(context, end);
+        RubyString i1 = isBeginless && !isEndless ? RubyString.newEmptyString(context.runtime) : inspectValue(context, begin).strDup(context.runtime);
+        RubyString i2 = isEndless && !isBeginless ? RubyString.newEmptyString(context.runtime) : inspectValue(context, end);
         i1.cat(DOTDOTDOT, 0, isExclusive ? 3 : 2);
         i1.append(i2);
-        i1.infectBy(i2);
-        i1.infectBy(this);
         return i1;
     }
 
@@ -376,8 +378,6 @@ public class RubyRange extends RubyObject {
         RubyString i2 = end.asString();
         i1.cat(DOTDOTDOT, 0, isExclusive ? 3 : 2);
         i1.append(i2);
-        i1.infectBy(i2);
-        i1.infectBy(this);
         return i1;
     }
 
@@ -386,12 +386,12 @@ public class RubyRange extends RubyObject {
         return getRuntime().newBoolean(isExclusive);
     }
     
-    @JRubyMethod(name = "eql?", required = 1)
+    @JRubyMethod(name = "eql?")
     public IRubyObject eql_p(ThreadContext context, IRubyObject other) {
         return equalityInner(context, other, MethodNames.EQL);
     }
 
-    @JRubyMethod(name = "==", required = 1)
+    @JRubyMethod(name = "==")
     public IRubyObject op_equal(ThreadContext context, IRubyObject other) {
         return equalityInner(context, other, MethodNames.OP_EQUAL);
     }
@@ -507,6 +507,7 @@ public class RubyRange extends RubyObject {
             while (rangeLt(context, v, end) != null) {
                 callback.doCall(context, v);
                 v = v.callMethod(context, "succ");
+                context.pollThreadEvents();
             }
         } else {
             IRubyObject c;
@@ -516,10 +517,20 @@ public class RubyRange extends RubyObject {
                     break;
                 }
                 v = v.callMethod(context, "succ");
+                context.pollThreadEvents();
             }
         }
     }
 
+    private boolean coverRangeP(ThreadContext context, IRubyObject val) {
+        if (begin.isNil() || rangeLess(context, begin, val) <= 0) {
+            int excl = isExclusive ? 1 : 0;
+            return end.isNil() || rangeLess(context, end, val) <= -excl;
+        }
+
+        return false;
+    }
+
     private boolean coverRange(ThreadContext context, RubyRange val) {
         int cmp;
 
@@ -559,28 +570,7 @@ public class RubyRange extends RubyObject {
 
         if (isEndless) throw runtime.newRangeError("cannot convert endless range to an array");
 
-        if (begin instanceof RubyFixnum && end instanceof RubyFixnum) {
-            long lim = ((RubyFixnum) end).getLongValue();
-            if (!isExclusive) {
-                lim++;
-            }
-
-            long base = ((RubyFixnum) begin).getLongValue();
-            long size = lim - base;
-            if (size > Integer.MAX_VALUE) {
-                throw runtime.newRangeError("Range size too large for to_a");
-            }
-            if (size < 0) {
-                return RubyArray.newEmptyArray(runtime);
-            }
-            IRubyObject[] array = new IRubyObject[(int) size];
-            for (int i = 0; i < size; i++) {
-                array[i] = RubyFixnum.newFixnum(runtime, base + i);
-            }
-            return RubyArray.newArrayMayCopy(runtime, array);
-        } else {
-            return RubyEnumerable.to_a(context, this);
-        }
+        return RubyEnumerable.to_a(context, this);
     }
 
     @Deprecated
@@ -598,7 +588,7 @@ public class RubyRange extends RubyObject {
         } else if (begin instanceof RubyFixnum && end instanceof RubyFixnum) {
             fixnumEach(context, block);
         } else if (begin instanceof RubySymbol && end instanceof RubySymbol) {
-            ((RubySymbol) begin).asString().uptoCommon(context, ((RubySymbol) end).asString(), isExclusive, block, true);
+            begin.asString().uptoCommon(context, end.asString(), isExclusive, block, true);
         } else {
             IRubyObject tmp = begin.checkStringType();
             if (!tmp.isNil()) {
@@ -611,7 +601,14 @@ public class RubyRange extends RubyObject {
                 if (!discreteObject(context, begin)) {
                     throw context.runtime.newTypeError("can't iterate from " + begin.getMetaClass().getName());
                 }
-                rangeEach(context, (ThreadContext ctx, IRubyObject arg) -> block.yield(ctx, arg));
+                if (!end.isNil()) {
+                    rangeEach(context, (ThreadContext ctx, IRubyObject arg) -> block.yield(ctx, arg));
+                } else {
+                    for (IRubyObject beg = begin;; beg = beg.callMethod(context, "succ")) {
+                        block.yield(context, beg);
+                        context.pollThreadEvents();
+                    }
+                }
             }
         }
         return this;
@@ -641,7 +638,7 @@ public class RubyRange extends RubyObject {
 
     @JRubyMethod(name = "step")
     public IRubyObject step(final ThreadContext context, final Block block) {
-        return block.isGiven() ? stepCommon(context, RubyFixnum.one(context.runtime), block) : step(context, context.nil, block);
+        return block.isGiven() ? stepCommon(context, RubyFixnum.one(context.runtime), block) : step(context, context.runtime.getNil(), block);
     }
 
     @JRubyMethod(name = "step")
@@ -670,7 +667,16 @@ public class RubyRange extends RubyObject {
     }
 
     private IRubyObject stepEnumeratorize(ThreadContext context, IRubyObject step, String method) {
-        if (begin instanceof RubyNumeric && (end.isNil() || end instanceof RubyNumeric)) {
+        if (!step.isNil() && !(step instanceof RubyNumeric)) {
+            step = step.convertToInteger("to_int");
+        }
+        if ((step instanceof RubyNumeric) && ((RubyNumeric) step).isZero()) {
+            throw context.runtime.newArgumentError("step can't be 0");
+        }
+
+        if ((begin instanceof RubyNumeric && (end.isNil() || end instanceof RubyNumeric)) ||
+                (end instanceof RubyNumeric && begin.isNil())) {
+
             return RubyArithmeticSequence.newArithmeticSequence(
                     context,
                     this,
@@ -721,8 +727,8 @@ public class RubyRange extends RubyObject {
                 b.uptoCommon(context, end.asString(), isExclusive, blockCallback);
             }
         } else if (begin instanceof RubyNumeric
-                || !TypeConverter.checkIntegerType(runtime, begin, "to_int").isNil()
-                || !TypeConverter.checkIntegerType(runtime, end, "to_int").isNil()) {
+                || !TypeConverter.checkToInteger(runtime, begin, "to_int").isNil()
+                || !TypeConverter.checkToInteger(runtime, end, "to_int").isNil()) {
             numericStep(context, runtime, step, block);
         } else {
             IRubyObject tmp = begin.checkStringType();
@@ -871,7 +877,6 @@ public class RubyRange extends RubyObject {
     }
 
     private static boolean discreteObject(ThreadContext context, IRubyObject obj) {
-        if (obj instanceof RubyTime) return false;
         return sites(context).respond_to_succ.respondsTo(context, obj, obj, false);
     }
 
@@ -902,9 +907,9 @@ public class RubyRange extends RubyObject {
 
     // MRI: r_cover_p
     private boolean rangeIncludes(ThreadContext context, IRubyObject val) {
-        if (rangeLess(context, begin, val) <= 0) {
+        if (isBeginless || rangeLess(context, begin, val) <= 0) {
             int excl = isExclusive ? 1 : 0;
-            if (end.isNil() || rangeLess(context, val, end) <= -excl) {
+            if (isEndless || rangeLess(context, val, end) <= -excl) {
                 return true;
             }
         }
@@ -915,6 +920,11 @@ public class RubyRange extends RubyObject {
 
     @JRubyMethod(frame = true)
     public IRubyObject min(ThreadContext context, Block block) {
+        return min(context, null, block);
+    }
+
+    @JRubyMethod(frame = true)
+    public IRubyObject min(ThreadContext context, IRubyObject arg, Block block) {
         if (begin.isNil()) {
             throw context.runtime.newRangeError("cannot get the minimum of beginless range");
         }
@@ -923,28 +933,34 @@ public class RubyRange extends RubyObject {
             if (end.isNil()) {
                 throw context.runtime.newRangeError("cannot get the minimum of endless range with custom comparison method");
             }
-            return Helpers.invokeSuper(context, this, block);
-        }
 
-        int cmp = isEndless ? -1 : RubyComparable.cmpint(context, invokedynamic(context, begin, MethodNames.OP_CMP, end), begin, end);
-        if (cmp > 0 || (cmp == 0 && isExclusive)) {
-            return context.nil;
-        }
+            return arg != null ? Helpers.invokeSuper(context, this, arg, block) : Helpers.invokeSuper(context, this, block);
+        } else if (arg != null) {
+            return first(context, arg);
+        } else {
+            int cmp = isEndless ? -1 : RubyComparable.cmpint(context, invokedynamic(context, begin, MethodNames.OP_CMP, end), begin, end);
+            if (cmp > 0 || (cmp == 0 && isExclusive)) {
+                return context.nil;
+            }
 
-        return begin;
+            return begin;
+        }
     }
 
     @JRubyMethod(frame = true)
     public IRubyObject max(ThreadContext context, Block block) {
         boolean isNumeric = end instanceof RubyNumeric;
 
-        if (isEndless) throw context.runtime.newRangeError("cannot get the maximum element of endless range");
+        if (isEndless) throw context.runtime.newRangeError("cannot get the maximum of endless range");
 
         if (block.isGiven() || (isExclusive && !isNumeric)) {
+            if (isBeginless) {
+                throw context.runtime.newRangeError("cannot get the maximum of beginless range with custom comparison method");
+            }
             return Helpers.invokeSuper(context, this, block);
         }
 
-        int cmp = RubyComparable.cmpint(context, invokedynamic(context, begin, MethodNames.OP_CMP, end), begin, end);
+        int cmp = isBeginless ? -1 : RubyComparable.cmpint(context, invokedynamic(context, begin, MethodNames.OP_CMP, end), begin, end);
         if (cmp > 0) {
             return context.nil;
         }
@@ -960,6 +976,7 @@ public class RubyRange extends RubyObject {
             if (!(begin instanceof RubyInteger)) {
                 throw context.runtime.newTypeError("cannot exclude end value with non Integer begin value");
             }
+
             if (end instanceof RubyFixnum) {
                 return RubyFixnum.newFixnum(context.runtime, ((RubyFixnum) end).getLongValue() - 1);
             }
@@ -970,13 +987,6 @@ public class RubyRange extends RubyObject {
         return end;
     }
 
-    @JRubyMethod(frame = true)
-    public IRubyObject min(ThreadContext context, IRubyObject arg, Block block) {
-        if (isEndless && block.isGiven()) throw context.runtime.newRangeError("cannot get the minimum of endless range with custom comparison method");
-        if (isEndless) return first(context, arg);
-        return Helpers.invokeSuper(context, this, arg, block);
-    }
-
     @JRubyMethod(frame = true)
     public IRubyObject max(ThreadContext context, IRubyObject arg, Block block) {
         if (isEndless) throw context.runtime.newRangeError("cannot get the maximum element of endless range");
@@ -985,7 +995,7 @@ public class RubyRange extends RubyObject {
 
     @JRubyMethod
     public IRubyObject first(ThreadContext context) {
-        return begin;
+        return first(context, null);
     }
 
     @JRubyMethod
@@ -995,6 +1005,10 @@ public class RubyRange extends RubyObject {
 
     @JRubyMethod
     public IRubyObject first(ThreadContext context, IRubyObject arg) {
+        if (isBeginless) throw context.runtime.newRangeError("cannot get the first element of beginless range");
+
+        if (arg == null) return begin;
+
         final Ruby runtime = context.runtime;
         final int num = RubyNumeric.num2int(arg);
         if (num < 0) {
@@ -1024,6 +1038,20 @@ public class RubyRange extends RubyObject {
         return result;
     }
 
+    @JRubyMethod
+    public IRubyObject count(ThreadContext context, Block block) {
+        if (isBeginless || isEndless) return RubyFloat.newFloat(context.runtime, RubyFloat.INFINITY);
+
+        return RubyEnumerable.count(context, this, block);
+    }
+
+    @JRubyMethod
+    public IRubyObject minmax(ThreadContext context, Block block) {
+        if (block.isGiven()) return Helpers.invokeSuper(context, this, context.runtime.getRange(), "minmax", NULL_ARRAY, block);
+
+        return RubyArray.newArray(context.runtime, callMethod("min"), callMethod("max"));
+    }
+
     @JRubyMethod
     public IRubyObject last(ThreadContext context) {
         if (isEndless) throw context.runtime.newRangeError("cannot get the last element of endless range");
@@ -1038,9 +1066,55 @@ public class RubyRange extends RubyObject {
     @JRubyMethod
     public IRubyObject last(ThreadContext context, IRubyObject arg) {
         if (isEndless) throw context.runtime.newRangeError("cannot get the last element of endless range");
+
+        if ((begin instanceof RubyInteger) && (end instanceof RubyInteger)
+            && this.getMetaClass().checkMethodBasicDefinition("each")) {
+                return intRangeLast(context, arg);
+        }
+
         return ((RubyArray) RubyKernel.new_array(context, this, this)).last(arg);
     }
 
+    // MRI rb_int_range_last
+    private RubyArray intRangeLast(ThreadContext context, IRubyObject arg) {
+        IRubyObject one = RubyInteger.int2fix(context.runtime, 1);
+        IRubyObject len1, len, nv, b;
+
+        len1 = ((RubyInteger)end).op_minus(context, begin);
+
+        if (isExclusive) {
+            end = ((RubyInteger)end).op_minus(context, one);
+            len = len1;
+        } else {
+            len = ((RubyInteger)len1).op_plus(context, one);
+        }
+
+        if (((RubyInteger)len).isZero() || Numeric.f_negative_p(context, (RubyInteger)len)) {
+            return RubyArray.newEmptyArray(context.runtime);
+        }
+
+        long n = RubyNumeric.num2long(arg);
+        if (n < 0) {
+            throw context.runtime.newArgumentError("negative array size");
+        }
+
+        nv = RubyInteger.int2fix(context.runtime, n);
+        if (Numeric.f_gt_p(context, nv, len)) {
+             nv = len;
+             n = RubyNumeric.num2long(nv);
+        }
+
+        RubyArray array = RubyArray.newArray(context.runtime, n);
+        b = ((RubyInteger)end).op_minus(context, nv);
+        while (n > 0) {
+            b = ((RubyInteger)b).op_plus(context, one);
+            array.push(b);
+            n--;
+        }
+
+        return array;
+    }
+
     @JRubyMethod
     public IRubyObject size(ThreadContext context) {
         if (begin instanceof RubyNumeric) {
@@ -1068,7 +1142,7 @@ public class RubyRange extends RubyObject {
             RubyRange range = (RubyRange) obj;
 
             marshalStream.registerLinkTarget(range);
-            List<Variable<Object>> attrs = range.getVariableList();
+            List<Variable<Object>> attrs = range.getMarshalVariableList();
 
             attrs.add(new VariableEntry<Object>("excl", range.isExclusive ? runtime.getTrue() : runtime.getFalse()));
             attrs.add(new VariableEntry<Object>("begin", range.begin));
@@ -1078,14 +1152,10 @@ public class RubyRange extends RubyObject {
         }
 
         @Override
-        public Object unmarshalFrom(Ruby runtime, RubyClass type,
-                UnmarshalStream unmarshalStream) throws IOException {
-            RubyRange range = (RubyRange) type.allocate();
+        public Object unmarshalFrom(Ruby runtime, RubyClass type, UnmarshalStream input) throws IOException {
+            RubyRange range = (RubyRange) input.entry(type.allocate());
 
-            unmarshalStream.registerLinkTarget(range);
-
-            // FIXME: Maybe we can just gank these off the line directly?
-            unmarshalStream.defaultVariablesUnmarshal(range);
+            input.ivar(null, range, null);
 
             IRubyObject excl = (IRubyObject) range.removeInternalVariable("excl");
             IRubyObject begin = (IRubyObject) range.removeInternalVariable("begin");
@@ -1162,8 +1232,8 @@ public class RubyRange extends RubyObject {
         IRubyObject _beg = sites.begin.call(context, range, range);
         IRubyObject _end = sites.end.call(context, range, range);
         boolean excludeEnd = sites.exclude_end.call(context, range, range).isTrue();
-        int beg = _beg.convertToInteger().getIntValue();
-        int end = _end.convertToInteger().getIntValue();
+        int beg = _beg.isNil() ? 0 : _beg.convertToInteger().getIntValue();
+        int end = _end.isNil() ? -1 :_end.convertToInteger().getIntValue();
         int origBeg = beg;
         int origEnd = end;
 
@@ -1199,6 +1269,36 @@ public class RubyRange extends RubyObject {
         return context.nil;
     }
 
+    public static class RangeLike {
+        final IRubyObject begin;
+        final IRubyObject end;
+        final boolean excl;
+
+        RangeLike(IRubyObject begin, IRubyObject end, boolean excl) {
+            this.begin = begin;
+            this.end = end;
+            this.excl = excl;
+        }
+
+        IRubyObject getRange(ThreadContext context) {
+            return Helpers.invoke(context, end, "-", begin);
+        }
+    }
+
+    // MRI: rb_range_values
+    public static RangeLike rangeValues(ThreadContext context, IRubyObject range) {
+        if (range instanceof RubyRange) {
+            RubyRange vrange = (RubyRange) range;
+            return new RangeLike(vrange.begin(context), vrange.end(context), vrange.isExcludeEnd());
+        } else if (range instanceof RubyArithmeticSequence) {
+            return null;
+        } else if (range.respondsTo("begin") && range.respondsTo("end") && range.respondsTo("exclude_end?")) {
+            return new RangeLike(Helpers.invoke(context, range, "begin"), Helpers.invoke(context, range, "end"), Helpers.invoke(context, range, "exclude_end?").isTrue());
+        }
+
+        return null;
+    }
+
     public static class BSearch {
         @JRubyMethod(meta = true)
         public static IRubyObject double_to_long_bits(ThreadContext context, IRubyObject bsearch, IRubyObject flote) {
