@@ -137,6 +137,8 @@ public class RubyPathname extends RubyObject {
     public static class PathnameKernelMethods {
         @JRubyMethod(name = "Pathname", module = true, visibility = Visibility.PRIVATE)
         public static IRubyObject newPathname(IRubyObject recv, IRubyObject path) {
+            if (path instanceof RubyPathname) return path;
+
             return RubyPathname.newInstance(recv.getRuntime().getCurrentContext(), path);
         }
     }
@@ -165,7 +167,6 @@ public class RubyPathname extends RubyObject {
             throw context.runtime.newArgumentError("pathname contains null byte");
         }
 
-        infectBy(str);
         this.setPath((RubyString) str.dup());
         return this;
     }
@@ -189,20 +190,6 @@ public class RubyPathname extends RubyObject {
         return super.freeze(context);
     }
 
-    @Override
-    @JRubyMethod
-    public IRubyObject taint(ThreadContext context) {
-        getPath().taint(context);
-        return super.taint(context);
-    }
-
-    @Override
-    @JRubyMethod
-    public IRubyObject untaint(ThreadContext context) {
-        getPath().untaint(context);
-        return super.untaint(context);
-    }
-
     @Override
     @JRubyMethod(name = { "==", "eql?" })
     public IRubyObject op_equal(ThreadContext context, IRubyObject other) {
@@ -269,7 +256,7 @@ public class RubyPathname extends RubyObject {
         return context.runtime.newString("#<Pathname:" + getPath() + ">");
     }
 
-    @JRubyMethod(required = 1, optional = 1, writes = BACKREF)
+    @JRubyMethod(required = 1, optional = 1, checkArity = false, writes = BACKREF)
     public IRubyObject sub(ThreadContext context, IRubyObject[] args, Block block) {
         IRubyObject result = sites(context).sub.call(context, this, getPath(), args, block);
         return newInstance(context, result);
@@ -284,10 +271,16 @@ public class RubyPathname extends RubyObject {
 
     /* Facade for File */
 
-    @JRubyMethod(alias = "fnmatch?", required = 1, optional = 1)
-    public IRubyObject fnmatch(ThreadContext context, IRubyObject[] args) {
-        args = insertPath(args, 1);
-        return context.runtime.getFile().callMethod(context, "fnmatch?", args);
+    @JRubyMethod(name = {"fnmatch", "fnmatch?"})
+    public IRubyObject fnmatch_p(ThreadContext context, IRubyObject arg0) {
+        RubyClass File = context.runtime.getFile();
+        return sites(context).fnmatch_p.call(context, File, File, arg0, getPath());
+    }
+
+    @JRubyMethod(name = {"fnmatch", "fnmatch?"})
+    public IRubyObject fnmatch_p(ThreadContext context, IRubyObject arg0, IRubyObject arg1) {
+        RubyClass File = context.runtime.getFile();
+        return sites(context).fnmatch_p.call(context, File, File, arg0, getPath(), arg1);
     }
 
     @JRubyMethod
@@ -304,7 +297,7 @@ public class RubyPathname extends RubyObject {
 
     /* Facade for IO */
 
-    @JRubyMethod(optional = 3)
+    @JRubyMethod(optional = 3, checkArity = false)
     public IRubyObject each_line(ThreadContext context, IRubyObject[] args, Block block) {
         return context.runtime.getIO().callMethod(context, "foreach", unshiftPath(args), block);
     }
@@ -316,7 +309,7 @@ public class RubyPathname extends RubyObject {
         return newInstance(context, context.runtime.getDir().callMethod("getwd"));
     }
 
-    @JRubyMethod(required = 1, optional = 1, meta = true)
+    @JRubyMethod(required = 1, optional = 2, checkArity = false, meta = true)
     public static IRubyObject glob(ThreadContext context, IRubyObject recv, IRubyObject[] args,
             Block block) {
         // TODO: yield block while iterating
@@ -330,15 +323,17 @@ public class RubyPathname extends RubyObject {
         }
     }
 
-    @JRubyMethod(required = 1, optional = 1)
+    @JRubyMethod(required = 1, optional = 1, checkArity = false)
     public IRubyObject glob(ThreadContext context, IRubyObject[] _args, Block block) {
+        int argc = Arity.checkArgumentCount(context, _args, 1, 2);
+
         Ruby runtime = context.runtime;
 
         IRubyObject[] args = new IRubyObject[3];
         boolean blockGiven = block.isGiven();
 
         args[0] = _args[0];
-        if (_args.length == 1) {
+        if (argc == 1) {
             args[1] = RubyFixnum.zero(runtime);
         } else {
             args[1] = _args[1];
@@ -442,4 +437,30 @@ public class RubyPathname extends RubyObject {
     private static JavaSites.PathnameSites sites(ThreadContext context) {
         return context.sites.Pathname;
     }
+
+    @Deprecated
+    @Override
+    @JRubyMethod
+    public IRubyObject taint(ThreadContext context) {
+        return this;
+    }
+
+    @Deprecated
+    @Override
+    @JRubyMethod
+    public IRubyObject untaint(ThreadContext context) {
+        return this;
+    }
+
+    @Deprecated
+    public IRubyObject fnmatch(ThreadContext context, IRubyObject[] args) {
+        switch (args.length) {
+            case 1:
+                return fnmatch_p(context, args[0]);
+            case 2:
+                return fnmatch_p(context, args[0], args[1]);
+            default:
+                throw context.runtime.newArgumentError(args.length, 1, 2);
+        }
+    }
 }
