@@ -104,6 +104,7 @@ public class RbConfigLibrary implements Library {
         if (architecture == null) architecture = "unknown";
         if ("amd64".equals(architecture)) architecture = "x86_64";
         if ("aarch64".equals(architecture) && Platform.IS_MAC) architecture = "arm64";
+        if ("loongarch64".equals(architecture)) architecture = "loongarch64";
 
         return architecture;
     }
@@ -348,8 +349,13 @@ public class RbConfigLibrary implements Library {
 
         if (Platform.IS_WINDOWS) {
             setConfig(context, CONFIG, "EXEEXT", ".exe");
+            setConfig(context, CONFIG, "SOEXT", "dll");
+        } else if (Platform.IS_MAC) {
+            setConfig(context, CONFIG, "EXEEXT", "");
+            setConfig(context, CONFIG, "SOEXT", "dylib");
         } else {
             setConfig(context, CONFIG, "EXEEXT", "");
+            setConfig(context, CONFIG, "SOEXT", "so");
         }
 
         setConfig(context, CONFIG, "ridir", newFile(shareDir, "ri").getPath());
