@@ -41,6 +41,7 @@ import org.jruby.RubyInteger;
 import org.jruby.RubyNumeric;
 import org.jruby.RubyString;
 import org.jruby.RubySymbol;
+import org.jruby.ast.util.ArgsUtil;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.runtime.Helpers;
 import org.jruby.runtime.ThreadContext;
@@ -72,6 +73,7 @@ import static jnr.constants.platform.ProtocolFamily.PF_INET;
 import static jnr.constants.platform.ProtocolFamily.PF_INET6;
 import static jnr.constants.platform.Sock.SOCK_DGRAM;
 import static jnr.constants.platform.Sock.SOCK_STREAM;
+import static org.jruby.ext.socket.Addrinfo.AI_CANONNAME;
 
 /**
  * Socket class methods for addresses, structures, and so on.
@@ -177,48 +179,45 @@ public class SocketUtils {
         final Ruby runtime = context.runtime;
         final List<IRubyObject> l = new ArrayList<IRubyObject>();
 
-        buildAddrinfoList(context, args, new AddrinfoCallback() {
-            @Override
-            public void addrinfo(InetAddress address, int port, Sock sock, Boolean reverse) {
-                boolean is_ipv6 = address instanceof Inet6Address;
-                boolean sock_stream = true;
-                boolean sock_dgram = true;
+        buildAddrinfoList(context, args, true, (address, port, sock, reverse, usesCanonical) -> {
+            boolean is_ipv6 = address instanceof Inet6Address;
+            boolean sock_stream = true;
+            boolean sock_dgram = true;
 
-                if (sock != null) {
-                    if (sock == SOCK_STREAM) {
-                        sock_dgram = false;
+            if (sock != null) {
+                if (sock == SOCK_STREAM) {
+                    sock_dgram = false;
 
-                    } else if (sock == SOCK_DGRAM) {
-                        sock_stream = false;
+                } else if (sock == SOCK_DGRAM) {
+                    sock_stream = false;
 
-                    }
                 }
+            }
 
-                IRubyObject[] c;
+            IRubyObject[] c;
 
-                if (sock_dgram) {
-                    c = new IRubyObject[7];
-                    c[0] = runtime.newString(is_ipv6 ? "AF_INET6" : "AF_INET");
-                    c[1] = runtime.newFixnum(port);
-                    c[2] = runtime.newString(getHostAddress(context, address, reverse));
-                    c[3] = runtime.newString(address.getHostAddress());
-                    c[4] = runtime.newFixnum(is_ipv6 ? PF_INET6 : PF_INET);
-                    c[5] = runtime.newFixnum(SOCK_DGRAM);
-                    c[6] = runtime.newFixnum(IPPROTO_UDP);
-                    l.add(RubyArray.newArrayMayCopy(runtime, c));
-                }
+            if (sock_dgram) {
+                c = new IRubyObject[7];
+                c[0] = runtime.newString(is_ipv6 ? "AF_INET6" : "AF_INET");
+                c[1] = runtime.newFixnum(port);
+                c[2] = runtime.newString(getHostAddress(context, address, reverse));
+                c[3] = runtime.newString(address.getHostAddress());
+                c[4] = runtime.newFixnum(is_ipv6 ? PF_INET6 : PF_INET);
+                c[5] = runtime.newFixnum(SOCK_DGRAM);
+                c[6] = runtime.newFixnum(IPPROTO_UDP);
+                l.add(RubyArray.newArrayMayCopy(runtime, c));
+            }
 
-                if (sock_stream) {
-                    c = new IRubyObject[7];
-                    c[0] = runtime.newString(is_ipv6 ? "AF_INET6" : "AF_INET");
-                    c[1] = runtime.newFixnum(port);
-                    c[2] = runtime.newString(getHostAddress(context, address, reverse));
-                    c[3] = runtime.newString(address.getHostAddress());
-                    c[4] = runtime.newFixnum(is_ipv6 ? PF_INET6 : PF_INET);
-                    c[5] = runtime.newFixnum(SOCK_STREAM);
-                    c[6] = runtime.newFixnum(IPPROTO_TCP);
-                    l.add(RubyArray.newArrayMayCopy(runtime, c));
-                }
+            if (sock_stream) {
+                c = new IRubyObject[7];
+                c[0] = runtime.newString(is_ipv6 ? "AF_INET6" : "AF_INET");
+                c[1] = runtime.newFixnum(port);
+                c[2] = runtime.newString(getHostAddress(context, address, reverse));
+                c[3] = runtime.newString(address.getHostAddress());
+                c[4] = runtime.newFixnum(is_ipv6 ? PF_INET6 : PF_INET);
+                c[5] = runtime.newFixnum(SOCK_STREAM);
+                c[6] = runtime.newFixnum(IPPROTO_TCP);
+                l.add(RubyArray.newArrayMayCopy(runtime, c));
             }
         });
 
@@ -229,35 +228,34 @@ public class SocketUtils {
         final Ruby runtime = context.runtime;
         final List<Addrinfo> l = new ArrayList<Addrinfo>();
 
-        buildAddrinfoList(context, args, new AddrinfoCallback() {
-            @Override
-            public void addrinfo(InetAddress address, int port, Sock sock, Boolean reverse) {
-                boolean sock_stream = true;
-                boolean sock_dgram = true;
+        buildAddrinfoList(context, args, false, (address, port, sock, reverse, usesCanonical) -> {
+            boolean sock_stream = true;
+            boolean sock_dgram = true;
 
-                if (sock != null) {
-                    if (sock == SOCK_STREAM) {
-                        sock_dgram = false;
+            if (sock != null) {
+                if (sock == SOCK_STREAM) {
+                    sock_dgram = false;
 
-                    } else if (sock == SOCK_DGRAM) {
-                        sock_stream = false;
+                } else if (sock == SOCK_DGRAM) {
+                    sock_stream = false;
 
-                    }
                 }
+            }
 
-                if (sock_dgram) {
-                    l.add(new Addrinfo(runtime, runtime.getClass("Addrinfo"),
-                            new InetSocketAddress(address, port),
-                            Sock.SOCK_DGRAM,
-                            SocketType.DATAGRAM));
-                }
+            if (sock_dgram) {
+                l.add(new Addrinfo(runtime, runtime.getClass("Addrinfo"),
+                        new InetSocketAddress(address, port),
+                        Sock.SOCK_DGRAM,
+                        SocketType.DATAGRAM,
+                        usesCanonical));
+            }
 
-                if (sock_stream) {
-                    l.add(new Addrinfo(runtime, runtime.getClass("Addrinfo"),
-                            new InetSocketAddress(address, port),
-                            Sock.SOCK_STREAM,
-                            SocketType.SOCKET));
-                }
+            if (sock_stream) {
+                l.add(new Addrinfo(runtime, runtime.getClass("Addrinfo"),
+                        new InetSocketAddress(address, port),
+                        Sock.SOCK_STREAM,
+                        SocketType.SOCKET,
+                        usesCanonical));
             }
         });
 
@@ -269,10 +267,12 @@ public class SocketUtils {
                 InetAddress address,
                 int port,
                 Sock sock,
-                Boolean reverse);
+                Boolean reverse,
+                boolean usesCanonical);
     }
 
-    public static void buildAddrinfoList(ThreadContext context, IRubyObject[] args, AddrinfoCallback callback) {
+    // FIXME: timeout is not actually implemented and while this original method dualed nice betwee Socket/AddrInfo they now deviate on 7th arg.
+    public static void buildAddrinfoList(ThreadContext context, IRubyObject[] args, boolean processLastArgAsReverse, AddrinfoCallback callback) {
         Ruby runtime = context.runtime;
         IRubyObject host = args[0];
         IRubyObject port = args[1];
@@ -284,9 +284,17 @@ public class SocketUtils {
         IRubyObject flags = args.length > 5 ? args[5] : context.nil;
         IRubyObject reverseArg = args.length > 6 ? args[6] : context.nil;
 
-        // The Ruby Socket.getaddrinfo function supports boolean/nil/Symbol values for the
-        // reverse_lookup parameter. We need to massage all valid inputs to true/false/null.
-        Boolean reverseLookup = RubyIPSocket.doReverseLookup(context, reverseArg);
+        Boolean reverseLookup = null;
+        IRubyObject timeout = context.nil;
+        if (processLastArgAsReverse) {
+            // The Ruby Socket.getaddrinfo function supports boolean/nil/Symbol values for the
+            // reverse_lookup parameter. We need to massage all valid inputs to true/false/null.
+             reverseLookup = RubyIPSocket.doReverseLookup(context, reverseArg);
+        } else {
+            if (reverseArg != context.nil) {
+                timeout = ArgsUtil.extractKeywordArg(context, "timeout", reverseArg);
+            }
+        }
 
         AddressFamily addressFamily = family.isNil() ? null : addressFamilyFromArg(family);
 
@@ -301,6 +309,8 @@ public class SocketUtils {
         // TODO: implement flags
         int flag = flags.isNil() ? 0 : RubyNumeric.fix2int(flags);
 
+        boolean displayCanonical = (flag & AI_CANONNAME) != 0;
+
         String hostString = null;
 
         // The value of 1 is for Socket::AI_PASSIVE.
@@ -319,7 +329,7 @@ public class SocketUtils {
                 // filter out unrelated address families if specified
                 if (addressFamily == AF_INET6 && !(addrs[i] instanceof Inet6Address)) continue;
                 if (addressFamily == AF_INET && !(addrs[i] instanceof Inet4Address)) continue;
-                callback.addrinfo(addrs[i], p, sock, reverseLookup);
+                callback.addrinfo(addrs[i], p, sock, reverseLookup, displayCanonical);
             }
 
         } catch(UnknownHostException e) {
