@@ -75,6 +75,8 @@ public abstract class DynamicMethod {
     protected final String name;
     /** An arbitrarily-typed "method handle" for use by compilers and call sites */
     protected Object handle;
+    /** How many times has this method been aliased. */
+    protected int aliasCount;
 
     private static final int BUILTIN_FLAG = 0b1;
     private static final int NOTIMPL_FLAG = 0b10;
@@ -623,4 +625,29 @@ public abstract class DynamicMethod {
     protected DynamicMethod(RubyModule implementationClass, Visibility visibility) {
         this(implementationClass, visibility, "(anonymous)");
     }
+
+    public void adjustAliasCount(int delta) {
+        this.aliasCount += delta;
+    }
+
+    public int getAliasCount() {
+        return this.aliasCount;
+    }
+
+    /**
+     * Indicates the method will behave like a ruby2 keywords accepting method.
+     * This must be a Ruby implementation to work.  See Module#ruby2_keywords
+     * for information on the semantics of a method which is marked this way.
+     */
+    public void setRuby2Keywords() {
+        // non-native (Ruby) methods implement this.  We have an empty impl here vs
+        // abstract because we are unsure if any external native extensions happen
+        // to implement their own methods.  If it were abstract that extension would
+        // no longer compile.
+        //
+        // If an external does have their own special method impl AND it is not
+        // a native (Java) method then it must override this method and handle state.
+        // Our IR subsystem will know by how it is marked to process arguments
+        // in an appropriate way.
+    }
 }
