@@ -33,32 +33,28 @@ import java.lang.reflect.InvocationTargetException;
 import java.util.Map;
 
 import org.jruby.Ruby;
-import org.jruby.RubyArray;
 import org.jruby.RubyClass;
 import org.jruby.RubyModule;
-import org.jruby.exceptions.RaiseException;
 import org.jruby.internal.runtime.AbstractIRMethod;
 import org.jruby.internal.runtime.SplitSuperState;
 import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.internal.runtime.methods.JavaMethod.JavaMethodNBlock;
+import org.jruby.ir.JIT;
 import org.jruby.java.invokers.RubyToJavaInvoker;
 import org.jruby.javasupport.Java;
 import org.jruby.javasupport.Java.JCreateMethod;
 import org.jruby.javasupport.Java.JCtorCache;
-import org.jruby.javasupport.JavaConstructor;
-import org.jruby.javasupport.JavaObject;
+import org.jruby.javasupport.JavaUtil;
 import org.jruby.javasupport.proxy.JavaProxyClass;
 import org.jruby.javasupport.proxy.JavaProxyConstructor;
 import org.jruby.javasupport.proxy.ReifiedJavaProxy;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.CallSite;
-import org.jruby.runtime.JavaInternalBlockBody;
 import org.jruby.runtime.MethodIndex;
-import org.jruby.runtime.ObjectAllocator;
-import org.jruby.runtime.Signature;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
+import org.jruby.runtime.callsite.CacheEntry;
 
 public class ConcreteJavaProxy extends JavaProxy {
 
@@ -209,12 +205,12 @@ public class ConcreteJavaProxy extends JavaProxy {
      */
     public static class StaticJCreateMethod extends JavaMethodNBlock {
 
-        private Constructor<? extends ReifiedJavaProxy> withBlock;
-        private DynamicMethod oldInit;
+        private final Constructor<? extends ReifiedJavaProxy> withBlock;
+        final DynamicMethod oldInit;
 
-        StaticJCreateMethod(RubyModule cls, Constructor<? extends ReifiedJavaProxy> withBlock2, DynamicMethod oldinit) {
-            super(cls, PUBLIC, "__jcreate_static!");
-            this.withBlock = withBlock2;
+        StaticJCreateMethod(RubyModule implClass, Constructor<? extends ReifiedJavaProxy> javaProxyConstructor, DynamicMethod oldinit) {
+            super(implClass, PUBLIC, "__jcreate_static!");
+            this.withBlock = javaProxyConstructor;
             this.oldInit = oldinit;
         }
 
@@ -236,10 +232,6 @@ public class ConcreteJavaProxy extends JavaProxy {
             return self;
         }
 
-        public DynamicMethod getOriginal() {
-            return oldInit;
-        }
-
         public static void tryInstall(Ruby runtime, RubyClass clazz, JavaProxyClass proxyClass,
                 Class<? extends ReifiedJavaProxy> reified, boolean overwriteInitialize) {
             try {
@@ -293,8 +285,8 @@ public class ConcreteJavaProxy extends JavaProxy {
             }
 
             if (ctor == null) {
-                JavaObject jo = (JavaObject) initialize.call(context, self, clazz, "new", args);
-                return ((ReifiedJavaProxy) jo.getValue()).___jruby$rubyObject();
+                ReifiedJavaProxy proxy = JavaUtil.unwrapJava(initialize.call(context, self, clazz, "new", args));
+                return proxy.___jruby$rubyObject();
             } else {
 
                 // assume no easy conversions, use ruby fallback.
@@ -319,7 +311,7 @@ public class ConcreteJavaProxy extends JavaProxy {
     }
 
     /**
-     * Used by reified classes, this class is tightly coupled with RealClassGenerator, splitInitialize, & finishInitialize
+     * Used by reified classes, this class is tightly coupled with RealClassGenerator, splitInitialize, &amp; finishInitialize
      * Do not refactor without looking at RCG
      */
     public static final class SplitCtorData {
@@ -329,42 +321,47 @@ public class ConcreteJavaProxy extends JavaProxy {
         
         // public field used by finishInitialized &  (ruby < ruby < java) generated classes
         public final IRubyObject[] rbarguments;
-        public Block blk = Block.NULL_BLOCK;
+        public final Block block;
+
+        // fields below are only used in ConcreteJavaProxy finishInitialize
+        private final AbstractIRMethod method;
+        private final String name;
+        private final SplitSuperState<?> state;
 
         /**
          * Picks and converts arguments for the super call
          * Leaves ctorIndex and arguments ready for the super call
          */
-        public SplitCtorData(IRubyObject[] args, JCtorCache cache, Ruby rt) {
+        SplitCtorData(Ruby runtime, IRubyObject[] args, JCtorCache cache) {
+            this(runtime, args, cache, null, null, null, Block.NULL_BLOCK);
+        }
+
+        SplitCtorData(Ruby runtime, IRubyObject[] args, JCtorCache cache,
+                      AbstractIRMethod method, SplitSuperState<?> state, Block block) {
+            this(runtime, args, cache, method, null, state, block);
+        }
+
+        SplitCtorData(Ruby runtime, IRubyObject[] args, JCtorCache cache,
+                      AbstractIRMethod method, String name, Block block) {
+            this(runtime, args, cache, method, name, null, block);
+        }
+
+        private SplitCtorData(Ruby runtime, IRubyObject[] args, JCtorCache cache,
+                              AbstractIRMethod method, String name, SplitSuperState<?> state, Block block) {
             rbarguments = args;
             if (cache == null) { // (ruby < ruby < java) super call from one IRO to another IRO ctor
                 ctorIndex = -1;
                 arguments = null;
             } else {
-                ctorIndex = JCreateMethod.forTypes(args, cache, rt);
+                ctorIndex = JCreateMethod.forTypes(runtime, args, cache);
                 arguments = RubyToJavaInvoker.convertArguments(cache.constructors[ctorIndex], args);
             }
-        }
 
-        public SplitCtorData(IRubyObject[] args, JCtorCache cache, Ruby rt, AbstractIRMethod air2, SplitSuperState<?> state2,
-                Block blk2) {
-            this(args, cache, rt);
-            air = air2;
-            state =state2;
-            blk = blk2;
+            this.method = method;
+            this.name = name;
+            this.state = state;
+            this.block = block;
         }
-
-        public SplitCtorData(IRubyObject[] args, JCtorCache cache, Ruby rt, AbstractIRMethod air2, String name2, Block blk2) {
-            this(args, cache, rt);
-            air = air2;
-            name = name2;
-            blk = blk2;
-        }
-
-        // fields below are only used in ConcreteJavaProxy finishInitialize
-        private AbstractIRMethod air = null;
-        private String name = null;
-        private SplitSuperState<?> state = null;
     }
 
     /**
@@ -372,40 +369,52 @@ public class ConcreteJavaProxy extends JavaProxy {
      * Do not refactor without looking at RCG
      * @return An object used by reified code and the finishInitialize method
      */
-    public SplitCtorData splitInitialized(RubyClass base, IRubyObject[] args, Block blk, JCtorCache jcc) {
-        String name = base.getClassConfig().javaCtorMethodName;
-        DynamicMethod dm = base.searchMethod(name);
-        if (dm != null && (dm instanceof StaticJCreateMethod)) dm = ((StaticJCreateMethod) dm).getOriginal();
-        DynamicMethod dm1 = base.searchMethodLateral(name); // only on ourself //TODO: missing default
+    public SplitCtorData splitInitialized(RubyClass base, IRubyObject[] args, Block block, JCtorCache jcc) {
+        final Ruby runtime = getRuntime();
+        final String name = base.getClassConfig().javaCtorMethodName;
+        final CacheEntry methodEntry = base.searchWithCache(name);
+        final boolean isLateral = isClassOrIncludedPrependedModule(methodEntry.sourceModule, base);
+        DynamicMethod method = methodEntry.method;
+        if (method instanceof StaticJCreateMethod) method = ((StaticJCreateMethod) method).oldInit;
 
         // jcreate is for nested ruby classes from a java class
-        if ((dm1 != null && !(dm instanceof InitializeMethod) && !(dm instanceof StaticJCreateMethod))) {
+        if (isLateral && method instanceof AbstractIRMethod) {
 
-            AbstractIRMethod air = (AbstractIRMethod) dm; // TODO: getMetaClass() ? or base? (below v)
+            AbstractIRMethod air = (AbstractIRMethod) method; // TODO: getMetaClass() ? or base? (below v)
 
-            SplitSuperState<?> state = air.startSplitSuperCall(getRuntime().getCurrentContext(), this, getMetaClass(),
-                    name, args, blk);
+            SplitSuperState<?> state = air.startSplitSuperCall(runtime.getCurrentContext(), this, getMetaClass(), name, args, block);
             if (state == null) { // no super in method
-                return new SplitCtorData(args, jcc, getRuntime(), air, name, blk);
-            } else {
-                return new SplitCtorData(state.callArrayArgs.toJavaArrayMaybeUnsafe(), jcc, getRuntime(), air, state, blk);
+                return new SplitCtorData(runtime, args, jcc, air, name, block);
             }
-        } else {
-            return new SplitCtorData(args, jcc, getRuntime());
+            return new SplitCtorData(runtime, state.callArrayArgs.toJavaArrayMaybeUnsafe(), jcc, air, state, block);
         }
+        return new SplitCtorData(runtime, args, jcc);
+    }
+
+    private static boolean isClassOrIncludedPrependedModule(final RubyModule methodSource, final RubyClass klass) {
+        if (methodSource == klass) return true;
+
+        RubyClass candidate = klass.getSuperClass();
+        while (candidate != null && (candidate.isIncluded() || candidate.isPrepended())) { // up till 'real' superclass
+            if (candidate == klass) return true;
+            candidate = candidate.getSuperClass();
+        }
+
+        return false;
     }
 
     /**
      * Used by reified classes, this method is tightly coupled with RealClassGenerator, splitInitialize
      * Do not refactor without looking at RCG
+     * <p>Note: invoked from generated byte-code</p>
      */
     public void finishInitialize(SplitCtorData returned) {
-        if (returned.air != null) {
+        if (returned.method != null) {
             if (returned.state != null) {
-                returned.air.finishSplitCall(returned.state);
+                returned.method.finishSplitCall(returned.state);
             } else { // no super, direct call
-                returned.air.call(getRuntime().getCurrentContext(), this, getMetaClass(),
-                        returned.name, returned.rbarguments, returned.blk);
+                returned.method.call(getRuntime().getCurrentContext(), this, getMetaClass(),
+                        returned.name, returned.rbarguments, returned.block);
             }
         }
         // Ignore other cases
