@@ -41,6 +41,7 @@ import org.jruby.RubyString;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.javasupport.JavaUtil;
+import org.jruby.runtime.Arity;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
@@ -244,7 +245,7 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
         }
 
         @Override
-        public <T> void visitAll(ThreadContext context, VisitorWithState visitor, T state) {
+        public <T> void visitAll(ThreadContext context, VisitorWithStateI visitor) {
             final Ruby runtime = getRuntime();
             // NOTE: this is here to make maps act similar to Hash-es which allow modifications while
             // iterating (meant from the same thread) ... thus we avoid iterating entrySet() directly
@@ -254,10 +255,16 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
             for ( Map.Entry entry : entries ) {
                 IRubyObject key = JavaUtil.convertJavaToUsableRubyObject(runtime, entry.getKey());
                 IRubyObject value = JavaUtil.convertJavaToUsableRubyObject(runtime, entry.getValue());
-                visitor.visit(context, this, key, value, index++, state);
+                visitor.visit(context, this, key, value, index++);
             }
         }
 
+        @Override
+        public <T> void visitAll(ThreadContext context, VisitorWithState visitor, T state) {
+            visitAll(context,
+                    (ctx, self, key, value, index) -> visitor.visit(ctx, self, key, value, index, state));
+        }
+
         @Override
         public boolean hasKey(IRubyObject key) {
             return mapDelegate().containsKey(key);
@@ -268,12 +275,6 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
             return mapDelegate().containsKey(key.toJava(Object.class)) ? context.tru : context.fals;
         }
 
-        @Override
-        @Deprecated
-        public RubyBoolean has_key_p(IRubyObject key) {
-            return has_key_p(getRuntime().getCurrentContext(), key);
-        }
-
         @Override
         public RubyBoolean has_value_p(ThreadContext context, IRubyObject val) {
             return mapDelegate().containsValue(val.toJava(Object.class)) ? context.tru : context.fals;
@@ -291,22 +292,8 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
             return RubyBoolean.newBoolean(context, mapDelegate() instanceof java.util.IdentityHashMap );
         }
 
-        @Override // re-invent @JRubyMethod(name = "any?")
-        public IRubyObject any_p(ThreadContext context, IRubyObject[] args, Block block) {
-            boolean patternGiven = args.length > 0;
-
-            if (isEmpty()) return context.fals;
-
-            if (!block.isGiven() && !patternGiven) return context.tru;
-            if (patternGiven) return any_p_p(context, args[0]);
-
-            if (block.getSignature().arityValue() > 1) {
-                return any_p_i_fast(context, block);
-            }
-            return any_p_i(context, block);
-        }
-
-        private RubyBoolean any_p_i(ThreadContext context, Block block) {
+        @Override
+        protected RubyBoolean any_p_i(ThreadContext context, Block block) {
             final Ruby runtime = context.runtime;
             for ( Map.Entry entry : entrySet() ) {
                 final IRubyObject key = JavaUtil.convertJavaToUsableRubyObject(runtime, entry.getKey());
@@ -318,7 +305,8 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
             return runtime.getFalse();
         }
 
-        private RubyBoolean any_p_i_fast(ThreadContext context, Block block) {
+        @Override
+        protected RubyBoolean any_p_i_fast(ThreadContext context, Block block) {
             final Ruby runtime = context.runtime;
             for ( Map.Entry entry : entrySet() ) {
                 final IRubyObject key = JavaUtil.convertJavaToUsableRubyObject(runtime, entry.getKey());
@@ -330,7 +318,8 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
             return runtime.getFalse();
         }
 
-        private RubyBoolean any_p_p(ThreadContext context, IRubyObject pattern) {
+        @Override
+        protected RubyBoolean any_p_p(ThreadContext context, IRubyObject pattern) {
             final Ruby runtime = context.runtime;
             for ( Map.Entry entry : entrySet() ) {
                 final IRubyObject key = JavaUtil.convertJavaToUsableRubyObject(runtime, entry.getKey());
@@ -385,6 +374,17 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
         @Override
         public final Set directEntrySet() { return entrySet(); }
 
+        @Override
+        protected void replaceWith(ThreadContext context, RubyHash otherHash) {
+            replaceExternally(context, otherHash);
+        }
+
+        @Deprecated
+        @Override
+        public IRubyObject any_p(ThreadContext context, IRubyObject[] args, Block block) {
+            return super.any_p(context, args, block);
+        }
+
     }
 
     @JRubyMethod(name = "default")
@@ -400,7 +400,7 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
     /** rb_hash_set_default
      *
      */
-    @JRubyMethod(name = "default=", required = 1)
+    @JRubyMethod(name = "default=")
     public IRubyObject default_value_set(ThreadContext context, final IRubyObject defaultValue) {
         return getOrCreateRubyHashMap(context.runtime).default_value_set(context, defaultValue);
     }
@@ -488,7 +488,7 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
     /** rb_hash_aset
      *
      */
-    @JRubyMethod(name = {"[]=", "store"}, required = 2)
+    @JRubyMethod(name = {"[]=", "store"})
     public IRubyObject op_aset(ThreadContext context, IRubyObject key, IRubyObject value) {
         return getOrCreateRubyHashMap(context.runtime).op_aset(context, key, value);
     }
@@ -512,27 +512,27 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
     /** rb_hash_aref
      *
      */
-    @JRubyMethod(name = "[]", required = 1)
+    @JRubyMethod(name = "[]")
     public IRubyObject op_aref(ThreadContext context, IRubyObject key) {
         return getOrCreateRubyHashMap(context.runtime).op_aref(context, key);
     }
 
-    @JRubyMethod(name = "<", required = 1)
+    @JRubyMethod(name = "<")
     public IRubyObject op_lt(ThreadContext context, IRubyObject other) {
         return getOrCreateRubyHashMap(context.runtime).op_lt(context, other);
     }
 
-    @JRubyMethod(name = "<=", required = 1)
+    @JRubyMethod(name = "<=")
     public IRubyObject op_le(ThreadContext context, IRubyObject other) {
         return getOrCreateRubyHashMap(context.runtime).op_le(context, other);
     }
 
-    @JRubyMethod(name = ">", required = 1)
+    @JRubyMethod(name = ">")
     public IRubyObject op_gt(ThreadContext context, IRubyObject other) {
         return getOrCreateRubyHashMap(context.runtime).op_gt(context, other);
     }
 
-    @JRubyMethod(name = ">=", required = 1)
+    @JRubyMethod(name = ">=")
     public IRubyObject op_ge(ThreadContext context, IRubyObject other) {
         return getOrCreateRubyHashMap(context.runtime).op_ge(context, other);
     }
@@ -561,7 +561,7 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
     /** rb_hash_has_key_p
      *
      */
-    @JRubyMethod(name = {"has_key?", "key?", "include?", "member?"}, required = 1)
+    @JRubyMethod(name = {"has_key?", "key?", "include?", "member?"})
     public RubyBoolean has_key_p(ThreadContext context, IRubyObject key) {
         return getOrCreateRubyHashMap(context.runtime).has_key_p(context, key);
     }
@@ -569,7 +569,7 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
     /** rb_hash_has_value
      *
      */
-    @JRubyMethod(name = {"has_value?", "value?"}, required = 1)
+    @JRubyMethod(name = {"has_value?", "value?"})
     public RubyBoolean has_value_p(ThreadContext context, IRubyObject expected) {
         return getOrCreateRubyHashMap(context.runtime).has_value_p(context, expected);
     }
@@ -619,7 +619,7 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
      */
     @JRubyMethod(name = "index")
     public IRubyObject index(ThreadContext context, IRubyObject expected) {
-        return getOrCreateRubyHashMap(context.runtime).index(context, expected);
+        return getOrCreateRubyHashMap(context.runtime).key(context, expected);
     }
 
     /** rb_hash_key
@@ -747,7 +747,7 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
     /** rb_hash_replace
      *
      */
-    @JRubyMethod(name = { "replace", "ruby_replace" }, required = 1) // collision with java.util.Map#replace on Java 8+
+    @JRubyMethod(name = { "replace", "ruby_replace" }) // collision with java.util.Map#replace on Java 8+
     public RubyHash replace(final ThreadContext context, IRubyObject other) {
         return getOrCreateRubyHashMap(context.runtime).replace(context, other);
     }
@@ -805,16 +805,36 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
         return dupImpl("clone");
     }
 
-    @JRubyMethod(name = "any?", optional = 1)
-    public IRubyObject any_p(ThreadContext context, IRubyObject[] args, Block block) {
-        return getOrCreateRubyHashMap(context.runtime).any_p(context, args, block);
+    @JRubyMethod(name = "any?")
+    public IRubyObject any_p(ThreadContext context, Block block) {
+        return getOrCreateRubyHashMap(context.runtime).any_p(context, block);
     }
 
-    @JRubyMethod(name = "dig", required = 1, rest = true)
+    @JRubyMethod(name = "any?")
+    public IRubyObject any_p(ThreadContext context, IRubyObject arg0, Block block) {
+        return getOrCreateRubyHashMap(context.runtime).any_p(context, arg0, block);
+    }
+
+    @JRubyMethod(name = "dig", required = 1, rest = true, checkArity = false)
     public IRubyObject dig(ThreadContext context, IRubyObject[] args) {
         return getOrCreateRubyHashMap(context.runtime).dig(context, args);
     }
 
+    @JRubyMethod(name = "dig")
+    public IRubyObject dig(ThreadContext context, IRubyObject arg0) {
+        return getOrCreateRubyHashMap(context.runtime).dig(context, arg0);
+    }
+
+    @JRubyMethod(name = "dig")
+    public IRubyObject dig(ThreadContext context, IRubyObject arg0, IRubyObject arg1) {
+        return getOrCreateRubyHashMap(context.runtime).dig(context, arg0, arg1);
+    }
+
+    @JRubyMethod(name = "dig")
+    public IRubyObject dig(ThreadContext context, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2) {
+        return getOrCreateRubyHashMap(context.runtime).dig(context, arg0, arg1, arg2);
+    }
+
     @SuppressWarnings("unchecked")
     private MapJavaProxy dupImpl(final String method) {
         final Map map = getMapObject();
@@ -850,4 +870,9 @@ public final class MapJavaProxy extends ConcreteJavaProxy {
         return getOrCreateRubyHashMap(context.runtime).sort(context, block);
     }
 
+    @Deprecated
+    public IRubyObject any_p(ThreadContext context, IRubyObject[] args, Block block) {
+        return getOrCreateRubyHashMap(context.runtime).any_p(context, args, block);
+    }
+
 }
