@@ -32,11 +32,12 @@ import org.jruby.*;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.anno.JRubyModule;
+import org.jruby.exceptions.NoMethodError;
 import org.jruby.internal.runtime.methods.JavaMethod;
 import org.jruby.java.proxies.ArrayJavaProxy;
+import org.jruby.java.util.ClassUtils;
 import org.jruby.javasupport.Java;
-import org.jruby.javasupport.JavaCallable;
-import org.jruby.javasupport.JavaClass;
+import org.jruby.runtime.Arity;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.Helpers;
 import org.jruby.runtime.JavaInternalBlockBody;
@@ -46,11 +47,13 @@ import org.jruby.runtime.backtrace.TraceType;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.RubyStringBuilder;
 
+import java.lang.reflect.AccessibleObject;
 import java.lang.reflect.Array;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 
+import static org.jruby.RubyEnumerator.enumeratorize;
 import static org.jruby.RubyModule.undefinedMethodMessage;
 import static org.jruby.javasupport.JavaUtil.convertJavaToUsableRubyObject;
 import static org.jruby.javasupport.JavaUtil.isJavaObject;
@@ -101,7 +104,7 @@ public abstract class JavaLang {
         public static IRubyObject each(final ThreadContext context, final IRubyObject self, final Block block) {
             final Ruby runtime = context.runtime;
             if ( ! block.isGiven() ) { // ... Enumerator.new(self, :each)
-                return runtime.getEnumerator().callMethod("new", self, runtime.newSymbol("each"));
+                return enumeratorize(context.runtime, self, "each");
             }
             java.lang.Iterable iterable = unwrapIfJavaObject(self);
             java.util.Iterator iterator = iterable.iterator();
@@ -115,8 +118,8 @@ public abstract class JavaLang {
         @JRubyMethod
         public static IRubyObject each_with_index(final ThreadContext context, final IRubyObject self, final Block block) {
             final Ruby runtime = context.runtime;
-            if ( ! block.isGiven() ) { // ... Enumerator.new(self, :each)
-                return runtime.getEnumerator().callMethod("new", self, runtime.newSymbol("each_with_index"));
+            if ( ! block.isGiven() ) { // ... Enumerator.new(self, :each_with_index)
+                return enumeratorize(context.runtime, self, "each_with_index");
             }
             java.lang.Iterable iterable = unwrapIfJavaObject(self);
             java.util.Iterator iterator = iterable.iterator();
@@ -476,8 +479,7 @@ public abstract class JavaLang {
             proxy.defineAlias("resource", "get_resource");
             proxy.defineAlias("declared_field", "get_declared_field");
             proxy.defineAlias("field", "get_field");
-            proxy.defineAlias("annotation", "get_annotation");
-            proxy.defineAlias("annotation_present?", "is_annotation_present");
+
             return proxy;
         }
 
@@ -577,10 +579,7 @@ public abstract class JavaLang {
         @JRubyMethod(name = "<=>") // Ruby Comparable
         public static IRubyObject cmp(final ThreadContext context, final IRubyObject self, final IRubyObject other) {
             final java.lang.Class that;
-            if ( other instanceof JavaClass ) {
-                that = ((JavaClass) other).getJavaClass();
-            }
-            else if ( isJavaObject(other) ) {
+            if ( isJavaObject(other) ) {
                 that = unwrapJavaObject(other);
             }
             else {
@@ -642,9 +641,15 @@ public abstract class JavaLang {
 
         // JavaClass facade (compatibility) :
 
-        @JRubyMethod(required = 1)
+        @JRubyMethod
         public static IRubyObject extend_proxy(final ThreadContext context, IRubyObject self, IRubyObject extender) {
-            return JavaClass.extend_proxy(context, self, extender);
+            java.lang.Class<?> klass = Java.unwrapClassProxy(self);
+            RubyModule proxy = Java.getProxyClass(context.runtime, klass);
+            try {
+                return extender.callMethod(context, "extend_proxy", proxy);
+            } catch (NoMethodError ex) {
+                throw context.runtime.newTypeError("proxy extender must have an extend_proxy method");
+            }
         }
 
         //@JRubyMethod(name = "simple_name") // (legacy) compatibility with JavaClass
@@ -653,13 +658,16 @@ public abstract class JavaLang {
         //    return context.runtime.newString(JavaClass.getSimpleName(klass));
         //}
 
-        @JRubyMethod(required = 1, rest = true)
+        @SuppressWarnings("deprecation")
+        @JRubyMethod(required = 1, rest = true, checkArity = false)
         public static IRubyObject java_method(ThreadContext context, IRubyObject self, final IRubyObject[] args) {
+            Arity.checkArgumentCount(context, args, 1, -1);
+
             final java.lang.Class klass = unwrapJavaObject(self);
 
             final java.lang.String methodName = args[0].asJavaString();
             try {
-                java.lang.Class<?>[] argumentTypes = JavaClass.getArgumentTypes(context, args, 1);
+                java.lang.Class<?>[] argumentTypes = ClassUtils.getArgumentTypes(context, args, 1);
                 final Method method = klass.getMethod(methodName, argumentTypes);
                 return Java.getInstance(context.runtime, method); // a JavaMethod like
             } catch (NoSuchMethodException e) {
@@ -668,13 +676,16 @@ public abstract class JavaLang {
             }
         }
 
-        @JRubyMethod(required = 1, rest = true)
+        @SuppressWarnings("deprecation")
+        @JRubyMethod(required = 1, rest = true, checkArity = false)
         public static IRubyObject declared_method(ThreadContext context, IRubyObject self, final IRubyObject[] args) {
+            Arity.checkArgumentCount(context, args, 1, -1);
+
             final java.lang.Class klass = unwrapJavaObject(self);
 
             final java.lang.String methodName = args[0].asJavaString();
             try {
-                java.lang.Class<?>[] argumentTypes = JavaClass.getArgumentTypes(context, args, 1);
+                java.lang.Class<?>[] argumentTypes = ClassUtils.getArgumentTypes(context, args, 1);
                 final Method method = klass.getDeclaredMethod(methodName, argumentTypes);
                 return Java.getInstance(context.runtime, method); // a JavaMethod like
             } catch (NoSuchMethodException e) {
@@ -683,29 +694,32 @@ public abstract class JavaLang {
             }
         }
 
-        @JRubyMethod(required = 1, rest = true)
+        @JRubyMethod(required = 1, rest = true, checkArity = false)
         public static IRubyObject declared_method_smart(ThreadContext context, IRubyObject self, final IRubyObject[] args) {
+            Arity.checkArgumentCount(context, args, 1, -1);
+
             final java.lang.Class klass = unwrapJavaObject(self);
 
             final java.lang.String methodName = args[0].asJavaString();
 
-            java.lang.Class<?>[] argumentTypes = JavaClass.getArgumentTypes(context, args, 1);
+            java.lang.Class<?>[] argumentTypes = ClassUtils.getArgumentTypes(context, args, 1);
 
-            JavaCallable callable = JavaClass.getMatchingCallable(context.runtime, klass, methodName, argumentTypes);
+            AccessibleObject callable = ClassUtils.getMatchingCallable(klass, methodName, argumentTypes);
 
             if ( callable != null ) {
-                return Java.getInstance(context.runtime, callable.accessibleObject()); // a JavaMethod or JavaConstructor like
+                return Java.getInstance(context.runtime, callable); // a JavaMethod or JavaConstructor like
             }
 
             final Ruby runtime = context.runtime;
             throw runtime.newNameError(undefinedMethodMessage(runtime, ids(runtime, methodName), ids(runtime, klass.getName()), false), methodName);
         }
 
+        @SuppressWarnings("deprecation")
         @JRubyMethod(rest = true)
         public static IRubyObject constructor(ThreadContext context, IRubyObject self, IRubyObject[] args) {
             final java.lang.Class klass = unwrapJavaObject(self);
             try {
-                java.lang.Class<?>[] parameterTypes = JavaClass.getArgumentTypes(context, args, 0);
+                java.lang.Class<?>[] parameterTypes = ClassUtils.getArgumentTypes(context, args, 0);
                 Constructor<?> constructor = klass.getConstructor(parameterTypes);
                 return Java.getInstance(context.runtime, constructor); // JavaConstructor like
             } catch (NoSuchMethodException e) {
@@ -714,11 +728,12 @@ public abstract class JavaLang {
             }
         }
 
+        @SuppressWarnings("deprecation")
         @JRubyMethod(rest = true)
         public static IRubyObject declared_constructor(ThreadContext context, IRubyObject self, IRubyObject[] args) {
             final java.lang.Class klass = unwrapJavaObject(self);
             try {
-                java.lang.Class<?>[] parameterTypes = JavaClass.getArgumentTypes(context, args, 0);
+                java.lang.Class<?>[] parameterTypes = ClassUtils.getArgumentTypes(context, args, 0);
                 Constructor<?> constructor = klass.getDeclaredConstructor(parameterTypes);
                 return Java.getInstance(context.runtime, constructor); // JavaConstructor like
             } catch (NoSuchMethodException e) {
@@ -734,7 +749,7 @@ public abstract class JavaLang {
             return Java.getInstance(context.runtime, arrayClass);
         }
 
-        @JRubyMethod(required = 1)
+        @JRubyMethod
         public static IRubyObject new_array(ThreadContext context, IRubyObject self, IRubyObject length) {
             final java.lang.Class klass = unwrapJavaObject(self);
 
