@@ -44,6 +44,7 @@ import org.jruby.util.RubyStringBuilder;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
 
+import static org.jruby.RubyEnumerator.enumeratorize;
 import static org.jruby.javasupport.JavaUtil.convertJavaArrayToRuby;
 import static org.jruby.javasupport.JavaUtil.convertJavaToUsableRubyObject;
 import static org.jruby.javasupport.JavaUtil.inspectObject;
@@ -201,7 +202,7 @@ public abstract class JavaUtil {
             return JavaLang.Iterable.count(context, self, obj, Block.NULL_BLOCK);
         } */
 
-        @JRubyMethod(name = "+", required = 1)
+        @JRubyMethod(name = "+")
         public static IRubyObject op_plus(final ThreadContext context, final IRubyObject self, final IRubyObject coll) {
             final IRubyObject dup = self.callMethod(context, "dup");
             java.util.Collection javaDup = unwrapIfJavaObject(dup);
@@ -214,7 +215,7 @@ public abstract class JavaUtil {
             return dup;
         }
 
-        @JRubyMethod(name = "-", required = 1)
+        @JRubyMethod(name = "-")
         public static IRubyObject op_minus(final ThreadContext context, final IRubyObject self, final IRubyObject coll) {
             final IRubyObject dup = self.callMethod(context, "dup");
             java.util.Collection javaDup = unwrapIfJavaObject(dup);
@@ -415,11 +416,11 @@ public abstract class JavaUtil {
             return Java.getInstance(context.runtime, list.subList(start, end));
         }
 
-        @JRubyMethod(name = "index", required = 0) // list.index { |val| val > 0 }
+        @JRubyMethod(name = "index") // list.index { |val| val > 0 }
         public static IRubyObject index(final ThreadContext context, final IRubyObject self, final Block block) {
             final Ruby runtime = context.runtime;
             if ( ! block.isGiven() ) { // list.index ... Enumerator.new(self, :index)
-                return runtime.getEnumerator().callMethod("new", self, runtime.newSymbol("index"));
+                return enumeratorize(context.runtime, self, "index");
             }
 
             final java.util.List list = unwrapIfJavaObject(self);
@@ -440,7 +441,7 @@ public abstract class JavaUtil {
             return context.nil;
         }
 
-        @JRubyMethod(name = "index", required = 1) // list.index '42'
+        @JRubyMethod(name = "index") // list.index '42'
         public static IRubyObject index(final ThreadContext context, final IRubyObject self, final IRubyObject val,
             final Block ignoredBlock) {
 
@@ -468,11 +469,11 @@ public abstract class JavaUtil {
             return context.nil;
         }
 
-        @JRubyMethod(name = "rindex", required = 0) // list.rindex { |val| val > 0 }
+        @JRubyMethod(name = "rindex") // list.rindex { |val| val > 0 }
         public static IRubyObject rindex(final ThreadContext context, final IRubyObject self, final Block block) {
             final Ruby runtime = context.runtime;
             if ( ! block.isGiven() ) { // list.rindex ... Enumerator.new(self, :rindex)
-                return runtime.getEnumerator().callMethod("new", self, runtime.newSymbol("rindex"));
+                return enumeratorize(context.runtime, self, "rindex");
             }
 
             final java.util.List list = unwrapIfJavaObject(self);
@@ -494,7 +495,7 @@ public abstract class JavaUtil {
             return context.nil;
         }
 
-        @JRubyMethod(name = "rindex", required = 1) // list.rindex '42'
+        @JRubyMethod(name = "rindex") // list.rindex '42'
         public static IRubyObject rindex(final ThreadContext context, final IRubyObject self, final IRubyObject val,
             final Block ignoredBlock) {
 
