@@ -5,6 +5,7 @@ import org.jcodings.EncodingDB;
 import org.jcodings.EncodingDB.Entry;
 import org.jcodings.specific.ASCIIEncoding;
 import org.jcodings.specific.ISO8859_16Encoding;
+import org.jcodings.specific.UTF8Encoding;
 import org.jcodings.spi.ISO_8859_16;
 import org.jcodings.util.CaseInsensitiveBytesHash;
 import org.jcodings.util.Hash.HashEntryIterator;
@@ -24,9 +25,7 @@ import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 import org.jcodings.specific.USASCIIEncoding;
-import org.jruby.RubyFixnum;
 import org.jruby.RubyString;
-import org.jruby.ext.nkf.RubyNKF;
 import org.jruby.util.SafePropertyAccessor;
 import org.jruby.util.io.EncodingUtils;
 
@@ -240,11 +239,7 @@ public final class EncodingService {
     public IRubyObject getDefaultExternal() {
         Encoding defaultEncoding = runtime.getDefaultExternalEncoding();
         if (defaultEncoding == null) {
-            // TODO: MRI seems to default blindly to US-ASCII and we were using Charset default from Java...which is right?
-            ByteList encodingName = ByteList.create("US-ASCII");
-            defaultEncoding = runtime.getEncodingService().loadEncoding(encodingName);
-
-            runtime.setDefaultExternalEncoding(defaultEncoding);
+            defaultEncoding = getLocaleEncoding();
         }
         return getEncoding(defaultEncoding);
     }
@@ -287,11 +282,6 @@ public final class EncodingService {
         if (arg instanceof RubyEncoding) {
             return ((RubyEncoding) arg).getEncoding();
         }
-        if (arg instanceof RubyFixnum) {
-            final int id = (int) arg.convertToInteger().getLongValue();
-            final String name = RubyNKF.NKFCharsetMap.get(id);
-            if ( name != null ) return getEncodingFromNKFName(name);
-        }
         if ( ( arg = arg.checkStringType() ).isNil() ) {
             return null;
         }
@@ -435,7 +425,7 @@ public final class EncodingService {
 
     private void checkAsciiEncodingName(ByteList name) {
         if (!name.getEncoding().isAsciiCompatible()) {
-            throw runtime.newArgumentError("invalid name encoding (non ASCII)");
+            throw runtime.newArgumentError("invalid encoding name (non ASCII)");
         }
     }
 
@@ -461,6 +451,11 @@ public final class EncodingService {
         return filesystemEncoding;
     }
 
+    // MRI: env_encoding
+    public Encoding getEnvEncoding() {
+        return Platform.IS_WINDOWS ? UTF8Encoding.INSTANCE : getLocaleEncoding();
+    }
+
     /**
      * Represents one of the four "special" internal encoding names: internal,
      * external, locale, or filesystem.
