@@ -74,18 +74,6 @@ public class LibrarySearcher {
         return expandedLoadPath.pathEntries;
     }
 
-    @Deprecated
-    public FoundLibrary findBySearchState(LoadService.SearchState state) {
-        FoundLibrary[] lib = {null};
-        char found = findLibraryForRequire(state.searchFile, lib);
-        if (found != 0) {
-            state.searchFile = lib[0].searchName;
-            state.library = lib[0];
-            state.setLoadName(lib[0].getLoadName());
-        }
-        return lib[0];
-    }
-
     // MRI: search_required
     public synchronized char findLibraryForRequire(String file, FoundLibrary[] path) {
         // check loaded features
@@ -267,7 +255,7 @@ public class LibrarySearcher {
                 || file.endsWith(".jar");
     }
 
-    private static class StringWrapper implements CharSequence {
+    protected static class StringWrapper implements CharSequence {
         private String str;
         private int beg;
         private int len;
@@ -331,7 +319,7 @@ public class LibrarySearcher {
 
         @Override
         public String toString() {
-            return str.substring(beg, beg + len);
+            return "StringWrapper(" + (str == null ? "" : str.substring(beg, beg + len)) + ")";
         }
 
         @Override
@@ -939,7 +927,7 @@ public class LibrarySearcher {
         }
 
         private LoadServiceResourceInputStream prepareInputStream(Ruby runtime) {
-            try (InputStream is = resource.inputStream()){
+            try (InputStream is = resource.openInputStream()){
                 return new LoadServiceResourceInputStream(is);
             } catch (IOException ioe) {
                 throw runtime.newLoadError("failure to load file: " + ioe.getLocalizedMessage(), searchName);
@@ -954,7 +942,7 @@ public class LibrarySearcher {
 
         @Override
         public void load(Ruby runtime, boolean wrap) {
-            try (InputStream ris = resource.inputStream()) {
+            try (InputStream ris = resource.openInputStream()) {
 
                 InputStream is = new BufferedInputStream(ris, 32768);
                 IRScope script = CompiledScriptLoader.loadScriptFromFile(runtime, is, null, scriptName, false);
