@@ -66,6 +66,7 @@ import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.runtime.encoding.MarshalEncoding;
 
 import static org.jruby.RubyBasicObject.getMetaClass;
+import static org.jruby.runtime.marshal.MarshalCommon.*;
 import static org.jruby.util.RubyStringBuilder.str;
 import static org.jruby.util.RubyStringBuilder.types;
 
@@ -78,17 +79,8 @@ public class MarshalStream extends FilterOutputStream {
     private final Ruby runtime;
     private final MarshalCache cache;
     private final int depthLimit;
-    private boolean tainted = false;
-
     private int depth = 0;
 
-    private final static char TYPE_IVAR = 'I';
-    private final static char TYPE_USRMARSHAL = 'U';
-    private final static char TYPE_USERDEF = 'u';
-    private final static char TYPE_UCLASS = 'C';
-    public final static String SYMBOL_ENCODING_SPECIAL = "E";
-    private final static String SYMBOL_ENCODING = "encoding";
-
     public MarshalStream(Ruby runtime, OutputStream out, int depthLimit) throws IOException {
         super(out);
 
@@ -105,7 +97,6 @@ public class MarshalStream extends FilterOutputStream {
         
         if (depth > depthLimit) throw runtime.newArgumentError("exceed depth limit");
 
-        tainted |= value.isTaint();
 
         writeAndRegister(value);
 
@@ -170,7 +161,7 @@ public class MarshalStream extends FilterOutputStream {
                     // object has instance vars and isn't a class, get a snapshot to be marshalled
                     // and output the ivar header here
 
-                    variables = value.getVariableList();
+                    variables = value.getMarshalVariableList();
 
                     // check if any of those variables were actually set
                     if (variables.size() > 0 || shouldMarshalEncoding(value)) {
@@ -389,7 +380,7 @@ public class MarshalStream extends FilterOutputStream {
 
         List<Variable<Object>> variables = null;
         if (marshaled.hasVariables()) {
-            variables = marshaled.getVariableList();
+            variables = marshaled.getMarshalVariableList();
             if (variables.size() > 0) {
                 write(TYPE_IVAR);
             } else {
@@ -548,12 +539,13 @@ public class MarshalStream extends FilterOutputStream {
         out.write(value);
     }
 
+    @Deprecated
     public boolean isTainted() {
-        return tainted;
+        return false;
     }
 
     @Deprecated
     public boolean isUntrusted() {
-        return tainted;
+        return false;
     }
 }
