@@ -30,6 +30,7 @@
 package org.jruby.util.cli;
 
 import org.jruby.RubyInstanceConfig;
+import org.jruby.common.RubyWarnings;
 import org.jruby.exceptions.MainExitException;
 import org.jruby.runtime.profile.builtin.ProfileOutput;
 import org.jruby.util.JRubyFile;
@@ -50,6 +51,8 @@ import java.util.Set;
 import java.util.HashSet;
 import java.util.function.BiFunction;
 import java.util.regex.Pattern;
+import java.util.stream.Collectors;
+import java.util.stream.Stream;
 
 /**
  * Encapsulated logic for processing JRuby's command-line arguments.
@@ -95,6 +98,7 @@ public class ArgumentProcessor {
     final RubyInstanceConfig config;
     private boolean endOfArguments = false;
     private int characterIndex = 0;
+    private boolean dashUpperU = false;
 
     private static final Pattern VERSION_FLAG = Pattern.compile("^--[12]\\.[89012]$");
 
@@ -239,6 +243,7 @@ public class ArgumentProcessor {
                 case 'd':
                     config.setDebug(true);
                     config.setVerbosity(RubyInstanceConfig.Verbosity.TRUE);
+                    config.getWarningCategories().add(RubyWarnings.Category.DEPRECATED);
                     break;
                 case 'e':
                     disallowedInRubyOpts(argument);
@@ -247,7 +252,11 @@ public class ArgumentProcessor {
                     config.setHasInlineScript(true);
                     break FOR;
                 case 'E':
-                    processEncodingOption(grabValue(getArgumentError("unknown encoding name")));
+                    if (dashUpperU) {
+                        MainExitException mee = fakeRuntimeError("jruby: default_internal already set to UTF-8");
+                        throw mee;
+                    }
+                    processEncodingOption("-E", grabValue(getArgumentError("unknown encoding name")));
                     break FOR;
                 case 'F':
                     disallowedInRubyOpts(argument);
@@ -267,12 +276,10 @@ public class ArgumentProcessor {
                     break FOR;
                 case 'I':
                     String s = grabValue(getArgumentError("-I must be followed by a directory name to add to lib path"));
-                    String separator = java.io.File.pathSeparator;
-                    if (":".equals(separator)) {
-                        separator = SEPARATOR;
-                    }
-                    String[] ls = s.split(separator);
-                    config.getLoadPaths().addAll(Arrays.asList(ls));
+                    String separator = ":".equals(File.pathSeparator) ? SEPARATOR : File.pathSeparator;
+                    Stream<String> paths = Arrays.stream(s.split(separator)).map(path -> new JRubyFile(path).getAbsolutePath());
+
+                    config.getLoadPaths().addAll(paths.collect(Collectors.toList()));
                     break FOR;
                 case 'J':
                     String js = grabOptionalValue();
@@ -285,14 +292,18 @@ public class ArgumentProcessor {
                 case 'K': // @Deprecated TODO no longer relevant in Ruby 2.x
                     String eArg = grabValue(getArgumentError("provide a value for -K"));
 
-                    config.setKCode(KCode.create(eArg));
+                    KCode kcode = KCode.create(eArg);
 
-                    // source encoding
-                    config.setSourceEncoding(config.getKCode().getEncoding().toString());
+                    if (kcode != KCode.NIL) {
+                        config.setKCode(kcode);
 
-                    // set external encoding if not already specified
-                    if (config.getExternalEncoding() == null) {
-                        config.setExternalEncoding(config.getKCode().getEncoding().toString());
+                        // source encoding
+                        config.setSourceEncoding(config.getKCode().getEncoding().toString());
+
+                        // set external encoding if not already specified
+                        if (config.getExternalEncoding() == null) {
+                            config.setExternalEncoding(config.getKCode().getEncoding().toString());
+                        }
                     }
 
                     break;
@@ -325,26 +336,25 @@ public class ArgumentProcessor {
                     disallowedInRubyOpts(argument);
                     runBinScript();
                     break FOR;
-                case 'T':
-                    {
-                        grabOptionalValue();
-                        break FOR;
-                    }
                 case 'U':
+                    dashUpperU = true;
                     config.setInternalEncoding("UTF-8");
                     break;
                 case 'v':
                     config.setVerbosity(RubyInstanceConfig.Verbosity.TRUE);
+                    config.getWarningCategories().add(RubyWarnings.Category.DEPRECATED);
                     config.setShowVersion(true);
                     break;
                 case 'w':
                     config.setVerbosity(RubyInstanceConfig.Verbosity.TRUE);
+                    config.getWarningCategories().add(RubyWarnings.Category.DEPRECATED);
                     break;
                 case 'W':
                     {
                         String temp = grabOptionalValue();
                         if (temp == null) {
                             config.setVerbosity(RubyInstanceConfig.Verbosity.TRUE);
+                            config.getWarningCategories().add(RubyWarnings.Category.DEPRECATED);
                         } else {
                             if (temp.equals("0")) {
                                 config.setVerbosity(RubyInstanceConfig.Verbosity.NIL);
@@ -352,10 +362,30 @@ public class ArgumentProcessor {
                                 config.setVerbosity(RubyInstanceConfig.Verbosity.FALSE);
                             } else if (temp.equals("2")) {
                                 config.setVerbosity(RubyInstanceConfig.Verbosity.TRUE);
+                                config.getWarningCategories().add(RubyWarnings.Category.DEPRECATED);
                             } else {
-                                MainExitException mee = new MainExitException(1, getArgumentError(" -W must be followed by either 0, 1, 2 or nothing"));
-                                mee.setUsageError(true);
-                                throw mee;
+                                switch (temp) {
+                                    case ":deprecated":
+                                        config.getWarningCategories().add(RubyWarnings.Category.DEPRECATED);
+                                        break;
+                                    case ":no-deprecated":
+                                        config.getWarningCategories().remove(RubyWarnings.Category.DEPRECATED);
+                                        break;
+                                    case ":experimental":
+                                        config.getWarningCategories().add(RubyWarnings.Category.EXPERIMENTAL);
+                                        break;
+                                    case ":no-experimental":
+                                        config.getWarningCategories().remove(RubyWarnings.Category.EXPERIMENTAL);
+                                        break;
+                                    default:
+                                        if (temp.charAt(0) == ':') {
+                                            config.getError().println("warning: unknown warning category: " + temp.substring(1));
+                                        } else {
+                                            MainExitException mee = new MainExitException(1, getArgumentError(" -W must be followed by either 0, 1, 2 or a valid category"));
+                                            mee.setUsageError(true);
+                                            throw mee;
+                                        }
+                                }
                             }
                         }
                         break FOR;
@@ -448,6 +478,21 @@ public class ArgumentProcessor {
                             config.getError().println("warning: unknown argument for --debug: `" + debug + "'");
                         }
                         break FOR;
+                    } else if (argument.startsWith("--encoding")) {
+                        String encodingValue;
+                        final int len = argument.length();
+                        if (len == "--encoding".length()) {
+                            characterIndex = len;
+                            encodingValue = grabValue(getArgumentError("unknown encoding name"));
+                        } else {
+                            int splitIndex = argument.indexOf('=');
+                            if (splitIndex == -1 || len == (splitIndex + 1)) {
+                                throw new MainExitException(0, "jruby: missing argument for --encoding");
+                            }
+                            encodingValue = argument.substring(splitIndex + 1);
+                        }
+                        processEncodingOption("--encoding", encodingValue);
+                        break FOR;
                     } else if (argument.equals("--jdb")) {
                         config.setDebug(true);
                         config.setVerbosity(RubyInstanceConfig.Verbosity.TRUE);
@@ -586,6 +631,28 @@ public class ArgumentProcessor {
                     } else if (argument.equals("--verbose")) {
                         config.setVerbosity(RubyInstanceConfig.Verbosity.TRUE);
                         break FOR;
+                    } else if (argument.startsWith("--backtrace-limit=")) {
+                        String limit = valueListFor(argument, "backtrace-limit")[0];
+                        config.setBacktraceLimit(Integer.parseInt(limit));
+                        break FOR;
+                    } else if (argument.startsWith("--external-encoding=")) {
+                        String externalEncoding = valueListFor(argument, "external-encoding")[0];
+                        config.setExternalEncoding(externalEncoding);
+                        break FOR;
+                    } else if (argument.startsWith("--external-encoding")) {
+                        characterIndex = argument.length();
+                        String externalEncoding = grabValue(getArgumentError("invalid encoding"));
+                        config.setExternalEncoding(externalEncoding);
+                        break FOR;
+                    } else if (argument.startsWith("--internal-encoding=")) {
+                        String internalEncoding = valueListFor(argument, "internal-encoding")[0];
+                        config.setInternalEncoding(internalEncoding);
+                        break FOR;
+                    } else if (argument.startsWith("--internal-encoding")) {
+                        characterIndex = argument.length();
+                        String internalEncoding = grabValue(getArgumentError("invalid encoding"));
+                        config.setInternalEncoding(internalEncoding);
+                        break FOR;
                     } else {
                         if (argument.equals("--")) {
                             // ruby interpreter compatibilty
@@ -621,10 +688,15 @@ public class ArgumentProcessor {
 
     private void disallowedInRubyOpts(CharSequence option) {
         if (rubyOpts) {
-            throw new MainExitException(1, "jruby: invalid switch in RUBYOPT: " + option + " (RuntimeError)");
+            throw fakeRuntimeError("jruby: invalid switch in RUBYOPT: " + option);
         }
     }
 
+    private MainExitException fakeRuntimeError(String message) {
+        // We process arguments before JRuby boots, so there's no RuntimeError class yet
+        return new MainExitException(1, message + " (RuntimeError)");
+    }
+
     private static void errorMissingEquals(String label) {
         MainExitException mee;
         mee = new MainExitException(1, "missing argument for --" + label + "\n");
@@ -632,11 +704,11 @@ public class ArgumentProcessor {
         throw mee;
     }
 
-    private void processEncodingOption(String value) {
+    private void processEncodingOption(String optionString, String value) {
         List<String> encodings = StringSupport.split(value, ':', 3);
         switch (encodings.size()) {
             case 3:
-                throw new MainExitException(1, "extra argument for -E: " + encodings.get(2));
+                throw new MainExitException(1, "extra argument for " + optionString + ": " + encodings.get(2));
             case 2:
                 config.setInternalEncoding(encodings.get(1));
             case 1:
