@@ -1190,9 +1190,10 @@ describe "A Ruby class implementing an interface returning a Java Object[]" do
         return a.to_java
       end
     end
-    expect(ArrayReturningInterfaceConsumer.new.eat(Bar.new)).not_to eq(nil)
-    expect(ArrayReturningInterfaceConsumer.new.eat(Bar.new).java_object.class.name).to eq('Java::JavaArray')
-    expect(ArrayReturningInterfaceConsumer.new.eat(Bar.new).java_object.class).to eq(Java::JavaArray)
+
+    obj = ArrayReturningInterfaceConsumer.new.eat(Bar.new)
+    expect( obj ).not_to be nil
+    expect( obj.java_object ).to be_a(Java::JavaArray) # ArrayJavaProxy (compat)
   end
 end
 
