@@ -9,7 +9,6 @@ describe "Random#bytes" do
     Random.new(33).bytes(2).should == Random.new(33).bytes(2)
   end
 
-  # Should double check this is official spec
   it "returns the same numeric output for a given seed across all implementations and platforms" do
     rnd = Random.new(33)
     rnd.bytes(2).should == "\x14\\"
@@ -18,7 +17,7 @@ describe "Random#bytes" do
   end
 
   it "returns the same numeric output for a given huge seed across all implementations and platforms" do
-    rnd = Random.new(bignum_value ** 4)
+    rnd = Random.new(2 ** (63 * 4))
     rnd.bytes(2).should == "_\x91"
     rnd.bytes(1000) # skip some
     rnd.bytes(2).should == "\x17\x12"
