@@ -97,6 +97,32 @@ describe "C-API String function" do
     end
   end
 
+  describe "rb_str_set_len on a UTF-16 String" do
+    before :each do
+      @str = "abcdefghij".force_encoding(Encoding::UTF_16BE)
+      # Make sure to unshare the string
+      @s.rb_str_modify(@str)
+    end
+
+    it "inserts two NULL bytes at the length" do
+      @s.rb_str_set_len(@str, 4).b.should == "abcd".b
+      @s.rb_str_set_len(@str, 8).b.should == "abcd\x00\x00gh".b
+    end
+  end
+
+  describe "rb_str_set_len on a UTF-32 String" do
+    before :each do
+      @str = "abcdefghijkl".force_encoding(Encoding::UTF_32BE)
+      # Make sure to unshare the string
+      @s.rb_str_modify(@str)
+    end
+
+    it "inserts four NULL bytes at the length" do
+      @s.rb_str_set_len(@str, 4).b.should == "abcd".b
+      @s.rb_str_set_len(@str, 12).b.should == "abcd\x00\x00\x00\x00ijkl".b
+    end
+  end
+
   describe "rb_str_buf_new" do
     it "returns the equivalent of an empty string" do
       buf = @s.rb_str_buf_new(10, nil)
@@ -108,7 +134,7 @@ describe "C-API String function" do
 
     it "returns a string with the given capacity" do
       buf = @s.rb_str_buf_new(256, nil)
-      @s.rb_str_capacity(buf).should == 256
+      @s.rb_str_capacity(buf).should >= 256
     end
 
     it "returns a string that can be appended to" do
@@ -181,12 +207,6 @@ describe "C-API String function" do
       @s.rb_str_new("hello", 3).should == "hel"
     end
 
-    ruby_version_is ''...'2.7' do
-      it "returns a non-tainted string" do
-        @s.rb_str_new("hello", 5).should_not.tainted?
-      end
-    end
-
     it "returns an empty string if len is 0" do
       @s.rb_str_new("hello", 0).should == ""
     end
@@ -335,24 +355,6 @@ describe "C-API String function" do
     end
   end
 
-  ruby_version_is ''...'2.7' do
-    describe "rb_tainted_str_new" do
-      it "creates a new tainted String" do
-        newstring = @s.rb_tainted_str_new("test", 4)
-        newstring.should == "test"
-        newstring.tainted?.should be_true
-      end
-    end
-
-    describe "rb_tainted_str_new2" do
-      it "creates a new tainted String" do
-        newstring = @s.rb_tainted_str_new2("test")
-        newstring.should == "test"
-        newstring.tainted?.should be_true
-      end
-    end
-  end
-
   describe "rb_str_append" do
     it "appends a string to another string" do
       @s.rb_str_append("Hello", " Goodbye").should == "Hello Goodbye"
@@ -378,6 +380,14 @@ describe "C-API String function" do
     it_behaves_like :string_times, :rb_str_times, -> str, times { @s.rb_str_times(str, times) }
   end
 
+  describe "rb_str_buf_append" do
+    it "concatenates a string to another string" do
+      str = "Your house "
+      @s.rb_str_buf_append(str, "is on fire?").should.equal?(str)
+      str.should == "Your house is on fire?"
+    end
+  end
+
   describe "rb_str_buf_cat" do
     it "concatenates a C string to a ruby string" do
       @s.rb_str_buf_cat("Your house is on fire").should == "Your house is on fire?"
@@ -603,9 +613,17 @@ describe "C-API String function" do
       filename = fixture(__FILE__, "read.txt")
       str = ""
       capacities = @s.RSTRING_PTR_read(str, filename)
-      capacities.should == [30, 53]
+      capacities[0].should >= 30
+      capacities[1].should >= 53
+      capacities[0].should < capacities[1]
       str.should == "fixture file contents to test read() with RSTRING_PTR"
     end
+
+    it "terminates the string with at least (encoding min length) \\0 bytes" do
+      @s.RSTRING_PTR_null_terminate("abc", 1).should == "\x00"
+      @s.RSTRING_PTR_null_terminate("abc".encode("UTF-16BE"), 2).should == "\x00\x00"
+      @s.RSTRING_PTR_null_terminate("abc".encode("UTF-32BE"), 4).should == "\x00\x00\x00\x00"
+    end
   end
 
   describe "RSTRING_LEN" do
@@ -655,22 +673,6 @@ describe "C-API String function" do
   end
 
   describe "SafeStringValue" do
-    ruby_version_is ''...'2.7' do
-      it "raises for tained string when $SAFE is 1" do
-        begin
-          Thread.new {
-            $SAFE = 1
-            -> {
-              @s.SafeStringValue("str".taint)
-            }.should raise_error(SecurityError)
-          }.join
-        ensure
-          $SAFE = 0
-        end
-      end
-
-      it_behaves_like :string_value_macro, :SafeStringValue
-    end
   end
 
   describe "rb_str_modify" do
@@ -682,27 +684,27 @@ describe "C-API String function" do
   describe "rb_str_modify_expand" do
     it "grows the capacity to bytesize + expand, not changing the bytesize" do
       str = @s.rb_str_buf_new(256, "abcd")
-      @s.rb_str_capacity(str).should == 256
+      @s.rb_str_capacity(str).should >= 256
 
       @s.rb_str_set_len(str, 3)
       str.bytesize.should == 3
       @s.RSTRING_LEN(str).should == 3
-      @s.rb_str_capacity(str).should == 256
+      @s.rb_str_capacity(str).should >= 256
 
       @s.rb_str_modify_expand(str, 4)
       str.bytesize.should == 3
       @s.RSTRING_LEN(str).should == 3
-      @s.rb_str_capacity(str).should == 7
+      @s.rb_str_capacity(str).should >= 7
 
       @s.rb_str_modify_expand(str, 1024)
       str.bytesize.should == 3
       @s.RSTRING_LEN(str).should == 3
-      @s.rb_str_capacity(str).should == 1027
+      @s.rb_str_capacity(str).should >= 1027
 
       @s.rb_str_modify_expand(str, 1)
       str.bytesize.should == 3
       @s.RSTRING_LEN(str).should == 3
-      @s.rb_str_capacity(str).should == 4
+      @s.rb_str_capacity(str).should >= 4
     end
 
     it "raises an error if the string is frozen" do
@@ -800,12 +802,6 @@ describe :rb_external_str_new, shared: true do
     x80 = [0x80].pack('C')
     @s.send(@method, "#{x80}abc").encoding.should == Encoding::BINARY
   end
-
-  ruby_version_is ''...'2.7' do
-    it "returns a tainted String" do
-      @s.send(@method, "abc").tainted?.should be_true
-    end
-  end
 end
 
 describe "C-API String function" do
@@ -885,13 +881,6 @@ describe "C-API String function" do
       s.should == x
       s.encoding.should equal(Encoding::EUC_JP)
     end
-
-    ruby_version_is ''...'2.7' do
-      it "returns a tainted String" do
-        s = @s.rb_external_str_new_with_enc("abc", 3, Encoding::US_ASCII)
-        s.tainted?.should be_true
-      end
-    end
   end
 
   describe "rb_locale_str_new" do
@@ -1051,9 +1040,11 @@ end
       @s.rb_sprintf3(true.class).should == s
     end
 
-    it "formats a TrueClass VALUE as 'true' if sign specified in format" do
-      s = 'Result: true.'
-      @s.rb_sprintf4(true.class).should == s
+    ruby_bug "#19167", ""..."3.2" do
+      it "formats a TrueClass VALUE as 'true' if sign specified in format" do
+        s = 'Result: TrueClass.'
+        @s.rb_sprintf4(true.class).should == s
+      end
     end
 
     it "truncates a string to a supplied precision if that is shorter than the string" do
@@ -1209,4 +1200,31 @@ end
       str.should == "test fmt 41 6 number"
     end
   end
+
+  describe "rb_str_locktmp" do
+    it "raises an error when trying to lock an already locked string" do
+      str = "test"
+      @s.rb_str_locktmp(str).should == str
+      -> { @s.rb_str_locktmp(str) }.should raise_error(RuntimeError, 'temporal locking already locked string')
+    end
+
+    it "locks a string so that modifications would raise an error" do
+      str = "test"
+      @s.rb_str_locktmp(str).should == str
+      -> { str.upcase! }.should raise_error(RuntimeError, 'can\'t modify string; temporarily locked')
+    end
+  end
+
+  describe "rb_str_unlocktmp" do
+    it "unlocks a locked string" do
+      str = "test"
+      @s.rb_str_locktmp(str)
+      @s.rb_str_unlocktmp(str).should == str
+      str.upcase!.should == "TEST"
+    end
+
+    it "raises an error when trying to unlock an already unlocked string" do
+      -> { @s.rb_str_unlocktmp("test") }.should raise_error(RuntimeError, 'temporal unlocking already unlocked string')
+    end
+  end
 end
