@@ -489,8 +489,8 @@ class TestFile < Test::Unit::TestCase
     assert_equal("", File.extname("."))
     assert_equal("", File.extname("/."))
     assert_equal("", File.extname(".."))
-    assert_equal("", File.extname(".foo."))
-    assert_equal("", File.extname("foo."))
+    assert_equal(".", File.extname(".foo."))
+    assert_equal(".", File.extname("foo."))
   end
 
   def test_fnmatch
@@ -655,6 +655,13 @@ class TestFile < Test::Unit::TestCase
     end
   end
 
+  # ensure that uri:file: and file: URIs are treated the same by File.directory?
+  def test_uri_file_directory_query
+    ["uri:file://", "uri:file:/#{Dir.pwd}"].each do |uri|
+      assert(File.directory?(uri), uri)
+    end
+  end
+
   def test_file_query # - file?
     assert(File.file?('test/jruby/test_file.rb'))
     assert(! File.file?('test'))
@@ -1342,16 +1349,6 @@ class TestFile < Test::Unit::TestCase
     end
   end
 
-  # JRUBY-5286
-  def test_file_path_is_tainted
-    filename = 'test.txt'
-    io = File.new(filename, 'w')
-    assert io.path.tainted?
-  ensure
-    io.close
-    File.unlink(filename)
-  end
-
   # jruby/jruby#2331
   def test_classpath_realpath
     assert_equal("classpath:/java/lang/String.class", File.realpath("classpath:/java/lang/String.class"))
