@@ -901,10 +901,10 @@ class TestHigherJavasupport < Test::Unit::TestCase
     compare_to_ignore_case concat contentEquals content_equals endsWith
     ends_with equals equalsIgnoreCase equals_ignore_case getBytes getChars
     getClass get_bytes get_chars get_class hashCode hash_code indexOf
-    index_of intern java_class java_object java_object= lastIndexOf last_index_of
+    index_of intern java_class java_object lastIndexOf last_index_of
     length matches notify notifyAll notify_all regionMatches region_matches replace
     replaceAll replaceFirst replace_all replace_first split startsWith starts_with
-    subSequence sub_sequence substring taint tainted? toCharArray toLowerCase
+    subSequence sub_sequence substring toCharArray toLowerCase
     toString toUpperCase to_char_array to_lower_case to_string
     to_upper_case trim wait]
 
@@ -1278,15 +1278,6 @@ class TestHigherJavasupport < Test::Unit::TestCase
     assert !defined?(OuterClass::PrivateInstanceInnerClass)
   end
 
-  # Test the new "import" syntax
-  def test_import
-
-    assert_nothing_raised {
-      import java.nio.ByteBuffer
-      ByteBuffer.allocate(10)
-    }
-  end
-
   def test_java_exception_handling
     list = ArrayList.new
     begin
@@ -1871,7 +1862,7 @@ CLASSDEF
       fail 'expected to raise'
     rescue ArgumentError => e
       msg = e.message
-      assert msg.start_with?("wrong number of arguments calling `length` (given 1, expected 0)"), msg
+      assert msg.start_with?("`length': wrong number of arguments (given 1, expected 0)"), msg
     end
 
     begin # array proxy class
@@ -1964,24 +1955,27 @@ CLASSDEF
     end
   end
 
-  # original report: https://jira.codehaus.org/browse/JRUBY-5582
-  # NOTE: we're not testing this "early" on still a good JI exercise
-  def test_set_security_manager
-    security_manager = java.lang.System.getSecurityManager
-    begin
-      java.lang.System.setSecurityManager( JRubySecurityManager.new )
-      assert java.lang.System.getSecurityManager.is_a?(JRubySecurityManager)
-      #puts java.lang.System.getSecurityManager.checked_perms.inspect
-    ensure
-      java.lang.System.setSecurityManager( security_manager )
+  # The Java security manager subsystem was deprecated in Java 17 and disabled in 18
+  if ENV_JAVA["java.specification.version"] < "18"
+    # original report: https://jira.codehaus.org/browse/JRUBY-5582
+    # NOTE: we're not testing this "early" on still a good JI exercise
+    def test_set_security_manager
+      security_manager = java.lang.System.getSecurityManager
+      begin
+        java.lang.System.setSecurityManager( JRubySecurityManager.new )
+        assert java.lang.System.getSecurityManager.is_a?(JRubySecurityManager)
+        #puts java.lang.System.getSecurityManager.checked_perms.inspect
+      ensure
+        java.lang.System.setSecurityManager( security_manager )
+      end
     end
-  end
 
-  class JRubySecurityManager < java.lang.SecurityManager
-    def initialize; @checked = [] end
-    def checked_perms; @checked end
+    class JRubySecurityManager < java.lang.SecurityManager
+      def initialize; @checked = [] end
+      def checked_perms; @checked end
 
-    def checkPermission( perm ); @checked << perm end
+      def checkPermission( perm ); @checked << perm end
+    end
   end
 
   def test_method_bind_edge_cases # GH-7244
